#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

# https://www.crowdsupply.com/fairwaves/xtrx

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk/Rst.
    ("clk60", 0, Pins("C16"), IOStandard("LVCMOS25")),

    # Leds.
    ("user_led", 0, Pins("N18"),  IOStandard("LVCMOS25")),

    # PCIe.
    ("pcie_x1", 0,
        Subsignal("rst_n", Pins("T3"), IOStandard("LVCMOS25"), Misc("PULLUP=TRUE")),
        Subsignal("clk_p", Pins("B8")),
        Subsignal("clk_n", Pins("A8")),
        Subsignal("rx_p",  Pins("B6")),
        Subsignal("rx_n",  Pins("A6")),
        Subsignal("tx_p",  Pins("B2")),
        Subsignal("tx_n",  Pins("A2")),
    ),

    ("pcie_x2", 0,
        Subsignal("rst_n", Pins("T3"), IOStandard("LVCMOS25"), Misc("PULLUP=TRUE")),
        Subsignal("clk_p", Pins("B8")),
        Subsignal("clk_n", Pins("A8")),
        Subsignal("rx_p",  Pins("B6 B4")),
        Subsignal("rx_n",  Pins("A6 A4")),
        Subsignal("tx_p",  Pins("B2 D2")),
        Subsignal("tx_n",  Pins("A2 D1")),
    ),

    # SPIFlash.
    ("flash_cs_n", 0, Pins("K19"), IOStandard("LVCMOS25")),
    ("flash", 0,
        Subsignal("mosi", Pins("D18")),
        Subsignal("miso", Pins("D19")),
        Subsignal("wp",   Pins("G18")),
        Subsignal("hold", Pins("F18")),
        IOStandard("LVCMOS25")
    ),

    # Power-Down.
    ("pwrdwn_n", 0, Pins("R19"), IOStandard("LVCMOS25")),

    # I2C buses.
    ("i2c", 0,
        Subsignal("scl", Pins("U14"), Misc("PULLUP=True")),
        Subsignal("sda", Pins("U15"), Misc("PULLUP=True")),
        IOStandard("LVCMOS25"),
    ),
    ("i2c", 1,
        Subsignal("scl", Pins("M1"), Misc("PULLUP=True")),
        Subsignal("sda", Pins("N1"), Misc("PULLUP=True")),
        IOStandard("LVCMOS33"),
    ),

    # GPS.
    ("gps", 0,
        Subsignal("rst",    Pins("L18"), IOStandard("LVCMOS25")),
        Subsignal("pps",    Pins("P3"),  Misc("PULLDOWN=True")),
        Subsignal("rx" ,    Pins("N2"),  Misc("PULLUP=True")),
        Subsignal("tx" ,    Pins("L1"),  Misc("PULLUP=True")),
        IOStandard("LVCMOS33")
    ),

    # VCTCXO.
    ("vctcxo", 0,
        Subsignal("sel",    Pins("V17"), Misc("PULLDOWN=True")),
        Subsignal("clk",    Pins("N17"), Misc("PULLDOWN=True")),
        IOStandard("LVCMOS25")
    ),

    # GPIO.
    ("gpio", 0,
        Subsignal("iovcc_sel",  Pins("V19")),
        Subsignal("en_smsigio", Pins("D17")),
        IOStandard("LVCMOS25")
    ),

    # RF-Switches / SKY13330, SKY13384.
    ("rf_switches", 0,
        Subsignal("tx", Pins("P1"),    Misc("PULLUP=True")),
        Subsignal("rx", Pins("K3 J3"), Misc("PULLUP=True")),
        IOStandard("LVCMOS33")
    ),

    # RF-IC / LMS7002M.
    ("lms7002m", 0,
        # Control.
        Subsignal("rst_n",    Pins("U19")),
        Subsignal("pwrdwn_n", Pins("W17")),
        Subsignal("rxen",     Pins("W18")),
        Subsignal("txen",     Pins("W19")),

        # SPI.
        Subsignal("clk",  Pins("W14")),
        Subsignal("cs_n", Pins("W13")),
        Subsignal("mosi", Pins("W16"), Misc("PULLDOWN=True")),
        Subsignal("miso", Pins("W15"), Misc("PULLDOWN=True")),

        # RX-Interface (LMS -> FPGA).
        Subsignal("diq1",   Pins("J19 H17 G17 K17 H19 U16 J17 P19 U17 N19 V15 V16")),
        Subsignal("txnrx1", Pins("M19")),
        Subsignal("iqsel1", Pins("P17")),
        Subsignal("mclk1",  Pins("L17")),
        Subsignal("fclk1",  Pins("G19")),

        # RX-Interface (FPGA -> LMS).
        Subsignal("diq2",   Pins("W2 U2 V3 V4 V5 W7 V2 W4 U5 V8 U7 U8")),
        Subsignal("txnrx2", Pins("U4")),
        Subsignal("iqsel2", Pins("U3")),
        Subsignal("mclk2",  Pins("W5")),
        Subsignal("fclk2",  Pins("W6")),

        # IOStandard/Slew Rate.
        IOStandard("LVCMOS25"),
        Misc("SLEW=FAST"),
    ),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk60"
    default_clk_period = 1e9/60e6

    def __init__(self, toolchain="vivado"):
        Xilinx7SeriesPlatform.__init__(self, "xc7a50tcpg236-2", _io, toolchain=toolchain)

        self.toolchain.bitstream_commands = [
            "set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]",
            "set_property BITSTREAM.CONFIG.CONFIGRATE 16 [current_design]",
            "set_property BITSTREAM.GENERAL.COMPRESS TRUE [current_design]"
        ]
        self.toolchain.additional_commands = [
            # Non-Multiboot SPI-Flash bitstream generation.
            "write_cfgmem -force -format bin -interface spix4 -size 16 -loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin",

            # Multiboot SPI-Flash Operational bitstream generation.
            "set_property BITSTREAM.CONFIG.TIMER_CFG 0x0001fbd0 [current_design]",
            "set_property BITSTREAM.CONFIG.CONFIGFALLBACK Enable [current_design]",
            "write_bitstream -force {build_name}_operational.bit ",
            "write_cfgmem -force -format bin -interface spix4 -size 16 -loadbit \"up 0x0 {build_name}_operational.bit\" -file {build_name}_operational.bin",

            # Multiboot SPI-Flash Fallback bitstream generation.
            "set_property BITSTREAM.CONFIG.NEXT_CONFIG_ADDR 0x00400000 [current_design]",
            "write_bitstream -force {build_name}_fallback.bit ",
            "write_cfgmem -force -format bin -interface spix4 -size 16 -loadbit \"up 0x0 {build_name}_fallback.bit\" -file {build_name}_fallback.bin"
        ]

    def create_programmer(self):
        return OpenOCD("openocd_xc7_ft232.cfg", "bscan_spi_xc7a50t.bit")

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk60", loose=True), 1e9/60e6)



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2022 Primesh Pinto <primeshp@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *

from litex.gen import *

from litex.build.generic_platform import *

from litex_boards.platforms import seeedstudio_spartan_edge_accelerator

from litex.soc.cores.clock import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser,WS2812
from litex.soc.cores.video import *

# Serial Port --------------------------------------------------------------------------------------

_serial_io = [
    # Use J10 connectors 0/1 IOs + GND.
    ("serial", 0,
        Subsignal("tx", Pins("j10:j10_0")),
        Subsignal("rx", Pins("j10:j10_1")),
        IOStandard("LVCMOS33")
    ),
]

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq, with_video_pll):
        self.rst       = Signal()
        self.cd_sys    = ClockDomain()
        self.cd_hdmi   = ClockDomain()
        self.cd_hdmi5x = ClockDomain()

        # # #

        clk100 = platform.request("clk100")
        rst_n  = platform.request("rst_n")

        self.pll = pll = S7PLL(speedgrade=-1)
        self.comb += pll.reset.eq(~rst_n | self.rst)
        pll.register_clkin(clk100, sys_clk_freq)
        pll.create_clkout(self.cd_sys, sys_clk_freq)
      
        if with_video_pll:
            self.video_pll = video_pll = S7PLL(speedgrade=-1)
            video_pll.reset.eq(~rst_n | self.rst)
            video_pll.register_clkin(clk100, 100e6)
            video_pll.create_clkout(self.cd_hdmi,   25e6)
            video_pll.create_clkout(self.cd_hdmi5x, 5*25e6)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    
    def __init__(self, sys_clk_freq=100e6,
        with_led_chaser     = True,
        with_jtagbone       = False,
        with_video_terminal = True,
        with_neopixel       = False,
        **kwargs):
        platform = seeedstudio_spartan_edge_accelerator.Platform()
        platform.add_extension(_serial_io)

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq, with_video_pll=with_video_terminal)

        # SoCCore ----------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, sys_clk_freq, ident = "LiteX SoC on Seeedstudio Spartan Edge Accelerator", **kwargs)

        # Jtagbone ---------------------------------------------------------------------------------
        if with_jtagbone:
            self.add_jtagbone()

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq
            )

        # Video ------------------------------------------------------------------------------------
        if with_video_terminal:
            self.videophy = VideoHDMIPHY(platform.request("hdmi"), clock_domain="hdmi")
            self.add_video_colorbars(phy=self.videophy, timings="640x480@75Hz", clock_domain="hdmi")
            #self.add_video_terminal(phy=self.videophy, timings="640x480@75Hz", clock_domain="hdmi") #Fixme Not enough BRAM
        
        # Neopixel ---------------------------------------------------------------------------------
        # To test Nexpixel with LiteX BIOS:
        # - mem_list (to get ws2812_base).
        # - mem_write <ws2812_base> 0x00100000
        if with_neopixel:
            self.ws2812 = WS2812(platform.request("rgb"), nleds=2, sys_clk_freq=sys_clk_freq)
            self.bus.add_slave(name="ws2812", slave=self.ws2812.bus, region=SoCRegion(size=2*4))

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=seeedstudio_spartan_edge_accelerator.Platform, description="LiteX SoC on Spartan Edge Accelerator.")
    parser.add_target_argument("--sys-clk-freq",        default=100e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--with-jtagbone",       action="store_true",       help="Enable Jtagbone support.")
    parser.add_target_argument("--with-video-terminal", action="store_true",       help="Enable Video Colorbars (HDMI).")
    parser.add_target_argument("--with-neopixel",       action="store_true",       help="Enable onboard 2 Neopixels Leds.")

    args = parser.parse_args()
    soc = BaseSoC(
        sys_clk_freq        = args.sys_clk_freq,
        with_jtagbone       = args.with_jtagbone,
        with_video_terminal = args.with_video_terminal,
        with_neopixel       = args.with_neopixel,
        **parser.soc_argdict
    )

    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

# Build/Use ----------------------------------------------------------------------------------------
#
# 1) SoC with regular UART and optional Ethernet connected to the CPU:
# Connect a USB/UART to J19: TX of the FPGA is DATA_LED-, RX of the FPGA is KEY+.
# ./colorlight_5a_75x.py --revision=7.0 (or 6.1)  --build (--with-ethernet to add Ethernet capability)
# Note: on revision 6.1, add --uart-baudrate=9600 to lower the baudrate.
# ./colorlight_5a_75x.py --load
# You should see the LiteX BIOS and be able to interact with it.
#
# 2) SoC with UART in crossover mode over Etherbone:
# ./colorlight_5a_75x.py --revision=7.0 (or 6.1) --uart-name=crossover --with-etherbone --csr-csv=csr.csv --build
# ./colorlight_5a_75x.py --load
# ping 192.168.1.50
# Get and install wishbone tool from: https://github.com/litex-hub/wishbone-utils/releases
# wishbone-tool --ethernet-host 192.168.1.50 --server terminal --csr-csv csr.csv
# You should see the LiteX BIOS and be able to interact with it.
#
# 3) SoC with USB-ACM UART (on V7.0):
# - Replace U23 with a SN74CBT3245APWR or remove U23 and place jumper wires to make the ports bi-directional.
# - Place a 15K resistor between J4 pin 2 and J4 pin 4.
# - Place a 15K resistor between J4 pin 3 and J4 pin 4.
# - Place a 1.5K resistor between J4 pin 1 and J4 pin 3.
# - Connect USB DP (Green) to J4 pin 3, USB DN (White) to J4 pin 2.
# ./colorlight_5a_75x.py --revision=7.0 --uart-name=usb_acm  --build
# ./colorlight_5a_75x.py --load
# You should see the LiteX BIOS and be able to interact with it.
#
# Note that you can also use a 5A-75E board:
# ./colorlight_5a_75x.py --board=5a-75e --revision=7.1 (or 6.0) --build
#
# Disclaimer: SoC 2) is still a Proof of Concept with large timings violations on the IP/UDP and
# Etherbone stack that need to be optimized. It was initially just used to validate the reversed
# pinout but happens to work on hardware...
#
# Note you can also use the i5a-907 board:
# ./colorlight_5a_75x.py --board=i5a-907 --revision=7.0 --build


from migen import *
from migen.genlib.resetsync import AsyncResetSynchronizer

from litex.gen import *

from litex.build.io import DDROutput

from litex_boards.platforms import colorlight_5a_75b, colorlight_5a_75e, colorlight_i5a_907

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser

from litedram.modules import M12L16161A, M12L64322A
from litedram.phy import GENSDRPHY, HalfRateGENSDRPHY

from liteeth.phy.ecp5rgmii import LiteEthPHYRGMII

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq, use_internal_osc=False, with_usb_pll=False, with_rst=True, sdram_rate="1:1"):
        self.rst    = Signal()
        self.cd_sys = ClockDomain()
        if sdram_rate == "1:2":
            self.cd_sys2x    = ClockDomain()
            self.cd_sys2x_ps = ClockDomain()
        else:
            self.cd_sys_ps = ClockDomain()

        # # #

        # Clk / Rst
        if not use_internal_osc:
            clk = platform.request("clk25")
            clk_freq = 25e6
        else:
            clk = Signal()
            div = 5
            self.specials += Instance("OSCG",
                                p_DIV = div,
                                o_OSC = clk)
            clk_freq = 310e6/div

        rst_n = 1 if not with_rst else platform.request("user_btn_n", 0)

        # PLL
        self.pll = pll = ECP5PLL()
        self.comb += pll.reset.eq(~rst_n | self.rst)
        pll.register_clkin(clk, clk_freq)
        pll.create_clkout(self.cd_sys,    sys_clk_freq)
        if sdram_rate == "1:2":
            pll.create_clkout(self.cd_sys2x,    2*sys_clk_freq)
            pll.create_clkout(self.cd_sys2x_ps, 2*sys_clk_freq, phase=180) # Idealy 90° but needs to be increased.
        else:
           pll.create_clkout(self.cd_sys_ps, sys_clk_freq, phase=180) # Idealy 90° but needs to be increased.

        # USB PLL
        if with_usb_pll:
            self.usb_pll = usb_pll = ECP5PLL()
            self.comb += usb_pll.reset.eq(~rst_n | self.rst)
            usb_pll.register_clkin(clk, clk_freq)
            self.cd_usb_12 = ClockDomain()
            self.cd_usb_48 = ClockDomain()
            usb_pll.create_clkout(self.cd_usb_12, 12e6, margin=0)
            usb_pll.create_clkout(self.cd_usb_48, 48e6, margin=0)

        # SDRAM clock
        sdram_clk = ClockSignal("sys2x_ps" if sdram_rate == "1:2" else "sys_ps")
        self.specials += DDROutput(1, 0, platform.request("sdram_clock"), sdram_clk)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, board, revision, sys_clk_freq=60e6, toolchain="trellis",
        with_ethernet    = False,
        with_etherbone   = False,
        with_uartbone    = False,
        eth_ip           = "192.168.1.50",
        eth_phy          = 0,
        with_led_chaser  = True,
        use_internal_osc = False,
        sdram_rate       = "1:1",
        with_spi_flash   = False,
        **kwargs):
        board = board.lower()
        assert board in ["5a-75b", "5a-75e", "i5a-907"]
        if board == "5a-75b":
            platform = colorlight_5a_75b.Platform(revision=revision, toolchain=toolchain)
        elif board == "5a-75e":
            platform = colorlight_5a_75e.Platform(revision=revision, toolchain=toolchain)
        elif board == "i5a-907":
            platform = colorlight_i5a_907.Platform(revision=revision, toolchain=toolchain)

        if board == "5a-75e" and revision == "6.0" and (with_etherbone or with_ethernet):
            assert use_internal_osc, "You cannot use the 25MHz clock as system clock since it is provided by the Ethernet PHY and will stop during PHY reset."

        # CRG --------------------------------------------------------------------------------------
        with_rst     = kwargs["uart_name"] not in ["serial", "crossover"] # serial_rx shared with user_btn_n.
        if board == "i5a-907":
            with_rst = True
        with_usb_pll = kwargs.get("uart_name", None) == "usb_acm"
        self.crg = _CRG(platform, sys_clk_freq,
            use_internal_osc = use_internal_osc,
            with_usb_pll     = with_usb_pll,
            with_rst         = with_rst,
            sdram_rate       = sdram_rate
        )

        # SoCCore ----------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, int(sys_clk_freq), ident="LiteX SoC on Colorlight " + board.upper(), **kwargs)

        # SDR SDRAM --------------------------------------------------------------------------------
        if not self.integrated_main_ram_size:
            sdrphy_cls = HalfRateGENSDRPHY if sdram_rate == "1:2" else GENSDRPHY
            self.sdrphy = sdrphy_cls(platform.request("sdram"), sys_clk_freq)
            if board == "5a-75e" and revision == "6.0":
                sdram_cls  = M12L64322A
            else:
                sdram_cls  = M12L16161A
            self.add_sdram("sdram",
                phy                     = self.sdrphy,
                module                  = sdram_cls(sys_clk_freq, sdram_rate),
                l2_cache_size           = kwargs.get("l2_size", 8192),
                l2_cache_full_memory_we = False,

            )

        # Ethernet / Etherbone ---------------------------------------------------------------------
        if with_ethernet or with_etherbone:
            self.ethphy = LiteEthPHYRGMII(
                clock_pads = self.platform.request("eth_clocks", eth_phy),
                pads       = self.platform.request("eth", eth_phy),
                tx_delay   = 0e-9)
            if with_ethernet:
                self.add_ethernet(phy=self.ethphy, data_width=32)
            if with_etherbone:
                self.add_etherbone(phy=self.ethphy, ip_address=eth_ip, data_width=32)

        # Leds -------------------------------------------------------------------------------------
        # Disable leds when serial is used.
        if (platform.lookup_request("serial", loose=True) is None and with_led_chaser
            or board == "i5a-907"):
            self.leds = LedChaser(
                pads         = platform.request_all("user_led_n"),
                sys_clk_freq = sys_clk_freq)

        # Uartbone ---------------------------------------------------------------------------------
        if with_uartbone:
            if board != "i5a-907":
                raise ValueError("uartbone only supported on i5a-907")
            self.add_uartbone(uart_name="uartbone")

        # SPI Flash --------------------------------------------------------------------------------
        if with_spi_flash:
            if board == "i5a-907":
                raise ValueError("SPI Flash chip is unknown on i5a-907, feel free to fix")
                # from litespi.modules import XXXXXX as SpiFlashModule
            elif board == "5a-75b" and revision == "6.0":
                raise ValueError("SPI Flash chip is unknown on 5A-75B v6.0, feel free to fix")
                # from litespi.modules import XXXXXX as SpiFlashModule
            elif board == "5a-75b" and revision == "6.1":
                # It's very possible that V6.0 uses this as well, but no documentation can be found for it
                from litespi.modules import GD25Q16C as SpiFlashModule
            # 5A-75B v7.0/v8.0 and all 5A-75Es seem to use W25Q32JV
            else:
                from litespi.modules import W25Q32JV as SpiFlashModule

            from litespi.opcodes import SpiNorFlashOpCodes
            self.mem_map["spiflash"] = 0x20000000
            self.add_spi_flash(mode="1x", module=SpiFlashModule(SpiNorFlashOpCodes.READ_1_1_1), with_master=False)


# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=colorlight_5a_75b.Platform, description="LiteX SoC on Colorlight 5A-75X.")
    parser.add_target_argument("--board",             default="5a-75b",         help="Board type (5a-75b, 5a-75e or i5a-907).")
    parser.add_target_argument("--revision",          default="7.0",            help="Board revision (6.0, 6.1, 7.0 or 8.0).")
    parser.add_target_argument("--sys-clk-freq",      default=60e6, type=float, help="System clock frequency.")
    ethopts = parser.target_group.add_mutually_exclusive_group()
    ethopts.add_argument("--with-ethernet",           action="store_true",    help="Enable Ethernet support.")
    ethopts.add_argument("--with-etherbone",          action="store_true",    help="Enable Etherbone support.")
    parser.add_target_argument("--with-uartbone",     action="store_true",    help="Add uartbone on 'FAN OUT' connector.")
    parser.add_target_argument("--eth-ip",            default="192.168.1.50", help="Ethernet/Etherbone IP address.")
    parser.add_target_argument("--eth-phy",           default=0, type=int,    help="Ethernet PHY (0 or 1).")
    parser.add_target_argument("--use-internal-osc",  action="store_true",    help="Use internal oscillator.")
    parser.add_target_argument("--sdram-rate",        default="1:1",          help="SDRAM Rate (1:1 Full Rate or 1:2 Half Rate).")
    parser.add_target_argument("--with-spi-flash",    action="store_true",    help="Add SPI flash support to the SoC")
    args = parser.parse_args()

    soc = BaseSoC(board=args.board, revision=args.revision,
        sys_clk_freq     = args.sys_clk_freq,
        toolchain        = args.toolchain,
        with_ethernet    = args.with_ethernet,
        with_etherbone   = args.with_etherbone,
        with_uartbone    = args.with_uartbone,
        eth_ip           = args.eth_ip,
        eth_phy          = args.eth_phy,
        use_internal_osc = args.use_internal_osc,
        sdram_rate       = args.sdram_rate,
        with_spi_flash   = args.with_spi_flash,
        **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)

    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram", ext=".svf")) # FIXME

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2022 Icenowy Zheng <icenowy@aosc.io>
# SPDX-License-Identifier: BSD-2-Clause

import os
from migen import *

from litex.gen import *

from litex_boards.platforms import sipeed_tang_nano_9k

from litex.soc.cores.clock.gowin_gw1n import GW1NPLL
from litex.soc.integration.soc_core import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser
from litex.soc.cores.video import *

from litex.soc.cores.hyperbus import HyperRAM

kB = 1024
mB = 1024*kB

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq, with_video_pll=False):
        self.rst    = Signal()
        self.cd_sys = ClockDomain()

        # # #

        # Clk / Rst
        clk27 = platform.request("clk27")
        rst_n = platform.request("user_btn", 0)

        # PLL
        self.pll = pll = GW1NPLL(devicename=platform.devicename, device=platform.device)
        self.comb += pll.reset.eq(~rst_n)
        pll.register_clkin(clk27, 27e6)
        pll.create_clkout(self.cd_sys, sys_clk_freq)

        # Video PLL
        if with_video_pll:
            self.video_pll = video_pll = GW1NPLL(devicename=platform.devicename, device=platform.device)
            self.comb += video_pll.reset.eq(~rst_n)
            video_pll.register_clkin(clk27, 27e6)
            self.cd_hdmi   = ClockDomain()
            self.cd_hdmi5x = ClockDomain()
            video_pll.create_clkout(self.cd_hdmi5x, 125e6)
            self.specials += Instance("CLKDIV",
                p_DIV_MODE= "5",
                i_RESETN = rst_n,
                i_HCLKIN = self.cd_hdmi5x.clk,
                o_CLKOUT = self.cd_hdmi.clk
            )

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=27e6, bios_flash_offset=0x0,
        with_led_chaser     = True,
        with_video_terminal = False,
        **kwargs):
        platform = sipeed_tang_nano_9k.Platform()

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq, with_video_pll=with_video_terminal)

        # SoCCore ----------------------------------------------------------------------------------
        # Disable Integrated ROM
        kwargs["integrated_rom_size"] = 0
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on Tang Nano 9K", **kwargs)

        # SPI Flash --------------------------------------------------------------------------------
        from litespi.modules import W25Q32
        from litespi.opcodes import SpiNorFlashOpCodes as Codes
        self.add_spi_flash(mode="1x", module=W25Q32(Codes.READ_1_1_1), with_master=False)

        # Add ROM linker region --------------------------------------------------------------------
        self.bus.add_region("rom", SoCRegion(
            origin = self.bus.regions["spiflash"].origin + bios_flash_offset,
            size   = 64*kB,
            linker = True)
        )
        self.cpu.set_reset_address(self.bus.regions["rom"].origin)

        # HyperRAM ---------------------------------------------------------------------------------
        if not self.integrated_main_ram_size:
            # TODO: Use second 32Mbit PSRAM chip.
            dq      = platform.request("IO_psram_dq")
            rwds    = platform.request("IO_psram_rwds")
            reset_n = platform.request("O_psram_reset_n")
            cs_n    = platform.request("O_psram_cs_n")
            ck      = platform.request("O_psram_ck")
            ck_n    = platform.request("O_psram_ck_n")
            class HyperRAMPads:
                def __init__(self, n):
                    self.clk   = Signal()
                    self.rst_n = reset_n[n]
                    self.dq    = dq[8*n:8*(n+1)]
                    self.cs_n  = cs_n[n]
                    self.rwds  = rwds[n]

            hyperram_pads = HyperRAMPads(0)
            self.comb += ck[0].eq(hyperram_pads.clk)
            self.comb += ck_n[0].eq(~hyperram_pads.clk)
            # FIXME: Issue with upstream HyperRAM core, so use old one. Need to investigate.
            if not os.path.exists("hyperbus.py"):
                os.system("wget https://github.com/litex-hub/litex-boards/files/8831568/hyperbus.py.txt")
                os.system("mv hyperbus.py.txt hyperbus.py")
            from hyperbus import HyperRAM
            self.hyperram = HyperRAM(hyperram_pads)
            self.bus.add_slave("main_ram", slave=self.hyperram.bus, region=SoCRegion(origin=self.mem_map["main_ram"], size=4*mB))

        # Video ------------------------------------------------------------------------------------
        if with_video_terminal:
            self.videophy = VideoGowinHDMIPHY(platform.request("hdmi"), clock_domain="hdmi")
            self.add_video_colorbars(phy=self.videophy, timings="640x480@60Hz", clock_domain="hdmi")
            #self.add_video_terminal(phy=self.videophy, timings="640x480@75Hz", clock_domain="hdmi") # FIXME: Free up BRAMs.


        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=sipeed_tang_nano_9k.Platform, description="LiteX SoC on Tang Nano 9K.")
    parser.add_target_argument("--flash",                action="store_true",      help="Flash Bitstream.")
    parser.add_target_argument("--sys-clk-freq",         default=27e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--bios-flash-offset",    default="0x0",            help="BIOS offset in SPI Flash.")
    parser.add_target_argument("--with-spi-sdcard",      action="store_true",      help="Enable SPI-mode SDCard support.")
    parser.add_target_argument("--with-video-terminal",  action="store_true",      help="Enable Video Terminal (HDMI).")
    parser.add_target_argument("--prog-kit",             default="openfpgaloader", help="Programmer select from Gowin/openFPGALoader.")
    args = parser.parse_args()

    soc = BaseSoC(
        sys_clk_freq        = args.sys_clk_freq,
        bios_flash_offset   = int(args.bios_flash_offset, 0),
        with_video_terminal = args.with_video_terminal,
        **parser.soc_argdict
    )

    if args.with_spi_sdcard:
        soc.add_spi_sdcard()

    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer(kit=args.prog_kit)
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

    if args.flash:
        prog = soc.platform.create_programmer(kit=args.prog_kit)
        prog.flash(0, builder.get_bitstream_filename(mode="flash", ext=".fs")) # FIXME
        # Axternal SPI programming not supported by gowin 'programmer_cli' now!
        # if needed, use openFPGALoader or Gowin programmer GUI
        if args.prog_kit == "openfpgaloader":
            prog.flash(int(args.bios_flash_offset, 0), builder.get_bios_filename(), external=True)

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Franck Jullien <franck.jullien@collshade.fr>
# Copyright (c) 2022 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.efinix.platform import EfinixPlatform
from litex.build.efinix import EfinixProgrammer

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk
    ("clk25",    0, Pins("B2"),  IOStandard("1.8_V_LVCMOS")),
    ("clk33",    0, Pins("P2"),  IOStandard("1.8_V_LVCMOS")),
    ("clk74_25", 0, Pins("A11"), IOStandard("1.8_V_LVCMOS")),

    # SD-Card
    ("spisdcard", 0,
        Subsignal("clk",  Pins("B12")),
        Subsignal("mosi", Pins("C12"), Misc("WEAK_PULLUP")),
        Subsignal("cs_n", Pins("A12"), Misc("WEAK_PULLUP")),
        Subsignal("miso", Pins("B14"), Misc("WEAK_PULLUP")),
        IOStandard("1.8_V_LVCMOS"),
    ),
    ("sdcard", 0,
        Subsignal("data", Pins("B14 A14 D12 A12"), Misc("WEAK_PULLUP")),
        Subsignal("cmd",  Pins("C12"), Misc("WEAK_PULLUP")),
        Subsignal("clk",  Pins("B12")),
        IOStandard("3.3_V_LVCMOS"),
    ),

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("R4")),
        Subsignal("rx", Pins("R3")),
        IOStandard("3.3_V_LVCMOS"), Misc("WEAK_PULLUP")
    ),

    # Leds
    ("user_led", 0,
        Subsignal("r", Pins("J15")),
        Subsignal("g", Pins("H10")),
        Subsignal("b", Pins("K14")),
        IOStandard("1.8_V_LVCMOS"),
    ),
    ("user_led", 1,
        Subsignal("r", Pins("H15")),
        Subsignal("g", Pins("H11")),
        Subsignal("b", Pins("J14")),
        IOStandard("1.8_V_LVCMOS"),
    ),

    # Buttons
    ("user_btn", 0, Pins("K13"), IOStandard("1.8_V_LVCMOS")),
    ("user_btn", 1, Pins("J13"), IOStandard("1.8_V_LVCMOS")),
    ("user_btn", 2, Pins("C5"),  IOStandard("1.8_V_LVCMOS")),
    ("user_btn", 3, Pins("R13"), IOStandard("1.8_V_LVCMOS")),

    # Switches
    ("user_sw", 0, Pins("F3"), IOStandard("1.8_V_LVCMOS")),
    ("user_sw", 1, Pins("E3"), IOStandard("1.8_V_LVCMOS")),

    # SPIFlash
    ("spiflash", 0,
        Subsignal("cs_n", Pins("P1")),
        Subsignal("clk",  Pins("N1")),
        Subsignal("mosi", Pins("M1")),
        Subsignal("miso", Pins("L1")),
        IOStandard("1.8_V_LVCMOS")
    ),

    # HyperRAM (X16)
    ("hyperram", 0,
        Subsignal("dq",  Pins(
            "B6 C6 A5 A6  F7  F8  E7  D7",
            "B9 A9 F9 E9 C10 D10 A10 B10"
        ), IOStandard("1.8_V_LVCMOS")),
        Subsignal("rwds",  Pins("B8 C8"), IOStandard("1.8_V_LVCMOS")),
        Subsignal("cs_n",  Pins("A8"),    IOStandard("1.8_V_LVCMOS")),
        Subsignal("rst_n", Pins("D5"),    IOStandard("1.8_V_LVCMOS")),
        Subsignal("clk",   Pins("B7"),    IOStandard("1.8_V_LVCMOS")),
        Misc("SLEWRATE=FAST")
    ),

    # MIPI
    ("mipi_tx", 0,
        Subsignal("clk",   Pins("D13"), IOStandard("1.8_V_LVCMOS")),
        Subsignal("data0", Pins("C15"), IOStandard("1.8_V_LVCMOS")),
        Subsignal("data1", Pins("D14"), IOStandard("1.8_V_LVCMOS")),
        Subsignal("data2", Pins("E14"), IOStandard("1.8_V_LVCMOS")),
        Subsignal("data3", Pins("E12"), IOStandard("1.8_V_LVCMOS")),
        Misc("SLEWRATE=FAST")
    ),

    # MIPI
    ("mipi_rx", 0,
        Subsignal("clk",   Pins("M15"), IOStandard("1.8_V_LVCMOS")),
        Subsignal("data0", Pins("K11"), IOStandard("1.8_V_LVCMOS")),
        Subsignal("data1", Pins("L13"), IOStandard("1.8_V_LVCMOS")),
        Misc("SLEWRATE=FAST")
    ),

    ("cam_i2c", 0,
        Subsignal("sda",   Pins("H4"), Misc("WEAK_PULLUP")),
        Subsignal("scl",   Pins("H5"), Misc("WEAK_PULLUP")),
        Subsignal("reset", Pins("R14")),
        IOStandard("1.8_V_LVCMOS")
    ),
]

iobank_info = [
            ("1A", "1.8 V LVCMOS"),
            ("1B", "1.8 V LVCMOS"),
            ("2A", "1.8 V LVCMOS"),
            ("2B", "1.8 V LVCMOS"),
            ("3A", "1.8 V LVCMOS"),
            ("3B", "1.8 V LVCMOS"),
            ("4A", "1.8 V LVCMOS"),
            ("4B", "1.8 V LVCMOS"),
            ("BL", "3.3 V LVCMOS"),
            ("BR", "1.8 V LVCMOS"),
            ("TL", "1.8 V LVCMOS"),
            ("TR", "3.3 V LVCMOS"),
]

# QSE Connectors -----------------------------------------------------------------------------------

_connectors = [
    ["P1",
        "---", # 0
        #3V3      5V     GND GND                 GND GND                 GND GND         ↓
        "--- H14 --- G14 --- --- F12 G13 E12 F13 --- --- E15 H13 E14 H12 --- --- C13 G15",
        #  1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17  18  19  20 ↑
        # 21  22  23  24  25  26  27  28  28  30  31  32  33  34  35  36  37  38  39  40 ↓
        "D13 F15 --- --- D15 G11 D14 F11 --- --- C14 N14 C15 P14 --- ---  K4  A4  J3  B5",
        #        GND GND                 GND GND                 GND GND                 ↑
    ],
    ["P2",
        "---", # 0
        #3V3      5V     GND GND                 GND GND                 GND GND         ↓
        "---  R9 ---  P9 --- --- L11 N10 K11 M10 --- --- L12 R10 L13 P10 --- --- M14 R12",
        #  1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17  18  19  20 ↑
        # 21  22  23  24  25  26  27  28  28  30  31  32  33  34  35  36  37  38  39  40 ↓
        "M15 R11 --- --- K10 P11 J10 P12 --- --- K12 N13 J12 P15 --- ---  H5  H4 P13 R14",
        #        GND GND                 GND GND                 GND GND                 ↑
    ],
    ["P3",
        "---", # 0
        #3V3      5V     GND GND                 GND GND                 GND GND         ↓
        "---  R5 ---  P5 --- ---  M7  R6  L7  P6 --- ---  R8  N6  P8  M6 --- ---  K7  R7",
        #  1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17  18  19  20 ↑
        # 21  22  23  24  25  26  27  28  28  30  31  32  33  34  35  36  37  38  39  40 ↓
        " L8  P7 --- ---  N8  L6  M8  K6 --- ---  M9  A3  L9  B3 --- --- E10  C3 F10  C4",
        #        GND GND                 GND GND                 GND GND                 ↑
    ],
]

# QSE Extensions -----------------------------------------------------------------------------------

def rgmii_ethernet_qse_ios(con, n=""):
    return [
        (f"eth{n}_clocks", 0,
            Subsignal("tx", Pins(f"{con}:26")),
            Subsignal("rx", Pins(f"{con}:2")),
            IOStandard("1.8_V_LVCMOS"),
        ),
        (f"eth{n}", 0,
            Subsignal("rx_ctl",  Pins(f"{con}:27")),
            Subsignal("rx_data", Pins(f"{con}:21 {con}:19 {con}:15 {con}:13")),
            Subsignal("tx_ctl",  Pins(f"{con}:20")),
            Subsignal("tx_data", Pins(f"{con}:16 {con}:14 {con}:10 {con}:8")),
            Subsignal("rst_n",   Pins(f"{con}:40")),
            Subsignal("mdc",     Pins(f"{con}:39")),
            Subsignal("mdio",    Pins(f"{con}:37")),
            IOStandard("1.8_V_LVCMOS"),
        ),
    ]

# Platform -----------------------------------------------------------------------------------------

class Platform(EfinixPlatform):
    default_clk_name   = "clk25"
    default_clk_period = 1e9/50e6

    def __init__(self, toolchain="efinity"):
        EfinixPlatform.__init__(self, "Ti60F225C3", _io, _connectors, iobank_info=iobank_info, toolchain=toolchain)

    def create_programmer(self):
        return EfinixProgrammer()

    def do_finalize(self, fragment):
        EfinixPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk25", loose=True), 1e9/50e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.efinix.platform import EfinixPlatform
from litex.build.efinix import EfinixProgrammer

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk
    ("clk26", 0, Pins("F11"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),
    ("clk50", 0, Pins("D13"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),

    # Leds
    ("user_led", 0, Pins("E1"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),
    ("user_led", 1, Pins("F2"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),

    # Buttons
    ("user_btn", 0, Pins("F1"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),
    ("user_btn", 1, Pins("G2"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("K5")), # J11:1
        Subsignal("rx", Pins("K6")), # J11:2
        IOStandard("3.3_V_LVTTL_/_LVCMOS"),
        Misc("WEAK_PULLUP"),
    ),

    # SPIFlash
    ("spiflash", 0,
        Subsignal("cs_n", Pins("L1")),
        Subsignal("clk",  Pins("K1")),
        Subsignal("mosi", Pins("J1")),
        Subsignal("miso", Pins("J2")),
        IOStandard("3.3_V_LVTTL_/_LVCMOS")
    ),
]

# Bank voltage ---------------------------------------------------------------------------------------

_bank_info = [
            ("1A",       "3.3 V LVTTL / LVCMOS"),
            ("1B_1C_1D", "3.3 V LVTTL / LVCMOS"),
            ("1E",       "1.8 V"),
            ("3A_3B",    "3.3 V LVTTL / LVCMOS"),
            ("3C_3D_3E", "3.3 V LVTTL / LVCMOS"),
            ("4A",       "3.3 V LVTTL / LVCMOS"),
            ("4B",       "3.3 V LVTTL / LVCMOS"),
            ("BR",       "1.2 V"),
            ("TL",       "1.2 V"),
            ("TR",       "1.2 V"),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = []

# Platform -----------------------------------------------------------------------------------------

class Platform(EfinixPlatform):
    default_clk_name   = "clk50"
    default_clk_period = 1e9/50e6

    def __init__(self, toolchain="efinity"):
        EfinixPlatform.__init__(self, "T20F169C4", _io, _connectors, iobank_info=_bank_info, toolchain=toolchain)

    def create_programmer(self):
        return EfinixProgrammer()

    def do_finalize(self, fragment):
        EfinixPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk50", loose=True), 1e9/50e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2015-2021 HDMI2USB/LiteX developers
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import XilinxSpartan6Platform

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk/Rst
    ("clk100",    0, Pins("L15"), IOStandard("LVCMOS33")),
    ("cpu_reset", 0, Pins("T15"), IOStandard("LVCMOS33")),

    # Serial
    ("serial", 0,
        Subsignal("rx", Pins("A16"), IOStandard("LVCMOS33")),
        Subsignal("tx", Pins("B16"), IOStandard("LVCMOS33")),
    ),

    # FX2
    ("fx2", 0,
        Subsignal("ifclk",    Pins("C10")),
        Subsignal("data",     Pins("A2 D6 C6 B3 A3 B4 A4 C5")),
        Subsignal("addr",     Pins("A14 B14"), Misc("DRIVE=12")),
        Subsignal("flaga",    Pins("B9"),  Misc("DRIVE=12")),
        Subsignal("flagb",    Pins("A9"),  Misc("DRIVE=12")),
        Subsignal("flagc",    Pins("C15"), Misc("DRIVE=12")),
        Subsignal("rd_n",     Pins("F13"), Misc("DRIVE=12")),
        Subsignal("wr_n",     Pins("E13")),
        Subsignal("oe_n",     Pins("A15"), Misc("DRIVE=12")),
        Subsignal("cs_n",     Pins("B2")),
        Subsignal("pktend_n", Pins("C4"), Misc("DRIVE=12")),
        IOStandard("LVCMOS33")
    ),

    # SPI Flash
    ("spiflash4x", 0,
        Subsignal("cs_n", Pins("V3")),
        Subsignal("clk",  Pins("R15")),
        Subsignal("dq",   Pins("T13", "R13", "T14", "V14")),
        IOStandard("LVCMOS33"),
        Misc("SLEW=FAST"),
    ),

    # Leds
    ("user_led", 0, Pins("U18"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("M14"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("N14"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("L14"), IOStandard("LVCMOS33")),
    ("user_led", 4, Pins("M13"), IOStandard("LVCMOS33")),
    ("user_led", 5, Pins("D4"),  IOStandard("LVCMOS33")),
    ("user_led", 6, Pins("P16"), IOStandard("LVCMOS33")),
    ("user_led", 7, Pins("N12"), IOStandard("LVCMOS33")),

    # Buttons
    ("user_btn", 0, Pins("N4"), IOStandard("LVCMOS18")), # North button
    ("user_btn", 1, Pins("P4"), IOStandard("LVCMOS18")), # East button
    ("user_btn", 2, Pins("P3"), IOStandard("LVCMOS18")), # South button
    ("user_btn", 3, Pins("F6"), IOStandard("LVCMOS18")), # West button
    ("user_btn", 4, Pins("F5"), IOStandard("LVCMOS18")), # Center button

    # Switches.
    ("user_sw", 0, Pins("A10"), IOStandard("LVCMOS33")),
    ("user_sw", 1, Pins("D14"), IOStandard("LVCMOS33")),
    ("user_sw", 2, Pins("C14"), IOStandard("LVCMOS33")),
    ("user_sw", 3, Pins("P15"), IOStandard("LVCMOS33")),
    ("user_sw", 4, Pins("P12"), IOStandard("LVCMOS33")),
    ("user_sw", 5, Pins("R5"),  IOStandard("LVCMOS33")),
    ("user_sw", 6, Pins("T5"),  IOStandard("LVCMOS33")),
    ("user_sw", 7, Pins("E4"),  IOStandard("LVCMOS18")),

    # GMII/MII Ethernet
    ("eth_clocks", 0,
        Subsignal("tx",  Pins("K16")),
        Subsignal("gtx", Pins("L12")),
        Subsignal("rx",  Pins("K15")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 0,
        Subsignal("rst_n",   Pins("G13")),
        Subsignal("int_n",   Pins("L16")),
        Subsignal("mdio",    Pins("N17")),
        Subsignal("mdc",     Pins("F16")),
        Subsignal("rx_dv",   Pins("F17")),
        Subsignal("rx_er",   Pins("F18")),
        Subsignal("rx_data", Pins("G16 H14 E16 F15 F14 E18 D18 D17")),
        Subsignal("tx_en",   Pins("H15")),
        Subsignal("tx_er",   Pins("G18")),
        Subsignal("tx_data", Pins("H16 H13 K14 K13 J13 G14 H12 K12")),
        Subsignal("col",     Pins("C17")),
        Subsignal("crs",     Pins("C18")),
        IOStandard("LVCMOS33")
    ),

    # DDR2 SDRAM.
    ("ddram_clock", 0,
        Subsignal("p", Pins("G3")),
        Subsignal("n", Pins("G1")),
        IOStandard("DIFF_SSTL18_II"), Misc("IN_TERM=NONE")
    ),
    ("ddram", 0,
        Subsignal("cke",   Pins("H7"), IOStandard("SSTL18_II")),
        Subsignal("ras_n", Pins("L5"), IOStandard("SSTL18_II")),
        Subsignal("cas_n", Pins("K5"), IOStandard("SSTL18_II")),
        Subsignal("we_n",  Pins("E3"), IOStandard("SSTL18_II")),
        Subsignal("ba",    Pins("F2 F1 E1"), IOStandard("SSTL18_II")),
        Subsignal("a", Pins(
            "J7 J6 H5 L7 F3 H4 H3 H6",
            "D2 D1 F4 D3 G6"),
            IOStandard("SSTL18_II")),
        Subsignal("dq", Pins(
            "L2 L1 K2 K1 H2 H1 J3 J1",
            "M3 M1 N2 N1 T2 T1 U2 U1"),
            IOStandard("SSTL18_II")),
        Subsignal("dqs",   Pins("P2 L4"), IOStandard("DIFF_SSTL18_II")),
        Subsignal("dqs_n", Pins("P1 L3"), IOStandard("DIFF_SSTL18_II")),
        Subsignal("dm",    Pins("K4 K3"), IOStandard("SSTL18_II")),
        Subsignal("odt",   Pins("K6"),    IOStandard("SSTL18_II"))
    ),

    # HDMI Out 0
    ("hdmi_out", 0,
        Subsignal("clk_p",   Pins("B6"), IOStandard("TMDS_33")),
        Subsignal("clk_n",   Pins("A6"), IOStandard("TMDS_33")),
        Subsignal("data0_p", Pins("D8"), IOStandard("TMDS_33")),
        Subsignal("data0_n", Pins("C8"), IOStandard("TMDS_33")),
        Subsignal("data1_p", Pins("C7"), IOStandard("TMDS_33")),
        Subsignal("data1_n", Pins("A7"), IOStandard("TMDS_33")),
        Subsignal("data2_p", Pins("B8"), IOStandard("TMDS_33")),
        Subsignal("data2_n", Pins("A8"), IOStandard("TMDS_33")),
        Subsignal("scl",     Pins("D9"), IOStandard("I2C")),
        Subsignal("sda",     Pins("C9"), IOStandard("I2C")),
    ),

    # HDMI In 0
    ("hdmi_in", 0,
        Subsignal("clk_p",   Pins("D11")),
        Subsignal("clk_n",   Pins("C11")),
        Subsignal("data0_p", Pins("G9")),
        Subsignal("data0_n", Pins("F9")),
        Subsignal("data1_p", Pins("B11")),
        Subsignal("data1_n", Pins("A11")),
        Subsignal("data2_p", Pins("B12")),
        Subsignal("data2_n", Pins("A12")),
        Subsignal("scl",     Pins("C13"), IOStandard("LVCMOS33")),
        Subsignal("sda",     Pins("A13"), IOStandard("LVCMOS33")),
    ),

    # HDMI In 1
    ("hdmi_in", 1,
        Subsignal("clk_p",   Pins("H17")),
        Subsignal("clk_n",   Pins("H18")),
        Subsignal("data0_p", Pins("K17")),
        Subsignal("data0_n", Pins("K18")),
        Subsignal("data1_p", Pins("L17")),
        Subsignal("data1_n", Pins("L18")),
        Subsignal("data2_p", Pins("J16")),
        Subsignal("data2_n", Pins("J18")),
        Subsignal("scl",     Pins("M16"), IOStandard("LVCMOS33")),
        Subsignal("sda",     Pins("M18"), IOStandard("LVCMOS33")),
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ("VHDCI",
        {
        "EXP-IO1_P"  : "U16",
        "EXP-IO2_P"  : "U15",
        "EXP-IO3_P"  : "U13",
        "EXP-IO4_P"  : "M11",
        "EXP-IO5_P"  : "R11",
        "EXP-IO6_P"  : "T12",
        "EXP-IO7_P"  : "N10",
        "EXP-IO8_P"  : "M10",
        "EXP-IO9_P"  : "U11",
        "EXP-IO10_P" : "R10",
        "EXP-IO11_P" : "U10",
        "EXP-IO12_P" :  "R8",
        "EXP-IO13_P" :  "M8",
        "EXP-IO14_P" :  "U8",
        "EXP-IO15_P" :  "U7",
        "EXP-IO16_P" :  "N7",
        "EXP-IO17_P" :  "T6",
        "EXP-IO18_P" :  "R7",
        "EXP-IO19_P" :  "N6",
        "EXP-IO20_P" :  "U5",
        "EXP-IO1_N"  : "V16",
        "EXP-IO2_N"  : "V15",
        "EXP-IO3_N"  : "V13",
        "EXP-IO4_N"  : "N11",
        "EXP-IO5_N"  : "T11",
        "EXP-IO6_N"  : "V12",
        "EXP-IO7_N"  : "P11",
        "EXP-IO8_N"  :  "N9",
        "EXP-IO9_N"  : "V11",
        "EXP-IO10_N" : "T10",
        "EXP-IO11_N" : "V10",
        "EXP-IO12_N" :  "T8",
        "EXP-IO13_N" :  "N8",
        "EXP-IO14_N" :  "V8",
        "EXP-IO15_N" :  "V7",
        "EXP-IO16_N" :  "P8",
        "EXP-IO17_N" :  "V6",
        "EXP-IO18_N" :  "T7",
        "EXP-IO19_N" :  "P7",
        "EXP-IO20_N" :  "V5",
        }
    ),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(XilinxSpartan6Platform):
    default_clk_name   = "clk100"
    default_clk_period = 1e9/100e6

    def __init__(self, toolchain="ise"):
        XilinxSpartan6Platform.__init__(self,  "xc6slx45-csg324-3", _io, _connectors, toolchain=toolchain)
        self.add_platform_command("""CONFIG VCCAUX="3.3";""")

    def create_programmer(self):
        return iMPACT()

    def do_finalize(self, fragment):
        XilinxSpartan6Platform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk100",           loose=True), 1e9/100e6)
        self.add_period_constraint(self.lookup_request("hdmi_in:clk_p", 0, loose=True), 1e9/74.25e6)
        self.add_period_constraint(self.lookup_request("hdmi_in:clk_p", 1, loose=True), 1e9/74.25e6)
        self.add_period_constraint(self.lookup_request("eth_clocks:rx",    loose=True), 1e9/25e6)
        self.add_period_constraint(self.lookup_request("fx2:ifclk",        loose=True), 1e9/100e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2022 JM Robles <roblesjm@gmail.com>
# Copyright (c) 2021 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.lattice import LatticeiCE40Platform
from litex.build.lattice.programmer import IceStormProgrammer

_io = [

    # Clock
    ("clk12", 0, Pins("49"), IOStandard("LVCMOS33")),
    # Leds
    ("user_leds", 0, Pins("45"), IOStandard("LVCMOS33")),
    ("user_leds", 1, Pins("44"), IOStandard("LVCMOS33")),
    ("user_leds", 2, Pins("43"), IOStandard("LVCMOS33")),
    ("user_leds", 3, Pins("42"), IOStandard("LVCMOS33")),
    ("user_leds", 4, Pins("41"), IOStandard("LVCMOS33")),
    ("user_leds", 5, Pins("39"), IOStandard("LVCMOS33")),
    ("user_leds", 6, Pins("38"), IOStandard("LVCMOS33")),
    ("user_leds", 7, Pins("37"), IOStandard("LVCMOS33")),
    # Switches
    ("sw1", 0, Pins("34"), IOStandard("LVCMOS33")),
    ("sw2", 0, Pins("33"), IOStandard("LVCMOS33")),

    # Serial
    ("serial", 0, 
        Subsignal("rx", Pins("62"), IOStandard("LVCMOS33")),
        Subsignal("tx", Pins("61"), IOStandard("LVCMOS33")),
    ),
    # SPI
    (
        "spiflash", 0, 
        Subsignal("cs_n", Pins("71")),
        Subsignal("clk", Pins("70")),
        Subsignal("mosi", Pins("67")),
        Subsignal("miso", Pins("68")),
        IOStandard("LVCMOS33")

    ),
    # ADC
    (
        "adc", 0,
        Subsignal("int", Pins(90)),
        Subsignal("sda", Pins(83)),
        Subsignal("scl", Pins(84)),
        IOStandard("LVCMOS33")
    )

]

_connectors = [
    ("d0", "2"),
    ("d1", "1"),
    ("d2", "4"),
    ("d3", "3"),
    ("d4", "8"),
    ("d5", "7"),
    ("d6", "10"),
    ("d7", "9"),
    ("d8", "20"),
    ("d9", "19"),
    ("d10", "22"),
    ("d11", "21"),
    ("d12", "63"),
    ("d13", "64"),

    ("a0", "114"),
    ("a1", "115"),
    ("a2", "116"),
    ("a3", "117"),
]

# Platform -------------------------------------------------------------------------------

class Platform(LatticeiCE40Platform):

    default_clk_name = "clk12"
    default_clk_period = 1e9/12e6

    def __init__(self, toolchain="icestorm", **kwargs):

        LatticeiCE40Platform.__init__(self, "ice40-hx8k-tq144:4k", _io, _connectors, toolchain=toolchain, **kwargs)

    def create_programmer(self, mode="direct"):

        return IceStormProgrammer()

    def do_finalize(self, fragment):
        LatticeiCE40Platform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk12", loose=True), 1e9/12e6)




#!/usr/bin/env python3
#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Antmicro <www.antmicro.com>
# SPDX-License-Identifier: BSD-2-Clause

import os
import math
import json

from migen import *

from litex.gen import *

from litex_boards.platforms import antmicro_datacenter_ddr4_test_board

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser
from litex.soc.cores.bitbang import I2CMaster
from litex.soc.cores.video import VideoS7HDMIPHY

from litedram.modules import MTA18ASF2G72PZ
from litedram.phy.s7ddrphy import A7DDRPHY
from litedram.init import get_sdram_phy_py_header
from litedram.core.controller import ControllerSettings
from litedram.common import PhySettings, GeomSettings, TimingSettings

from liteeth.phy import LiteEthS7PHYRGMII
from litex.soc.cores.hyperbus import HyperRAM

from litespi.modules import S25FL128S0
from litespi.opcodes import SpiNorFlashOpCodes as Codes

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq, iodelay_clk_freq, with_video_pll=False):
        self.rst          = Signal()
        self.cd_sys       = ClockDomain()
        self.cd_sys2x     = ClockDomain()
        self.cd_sys4x     = ClockDomain()
        self.cd_sys4x_dqs = ClockDomain()
        self.cd_idelay    = ClockDomain()

        self.cd_hdmi      = ClockDomain()
        self.cd_hdmi5x    = ClockDomain()

        # # #

        # Clk.
        clk100 = platform.request("clk100")

        self.pll = pll = S7PLL(speedgrade=-1)
        self.comb += pll.reset.eq(self.rst)
        pll.register_clkin(clk100, 100e6)
        pll.create_clkout(self.cd_sys,       sys_clk_freq)
        pll.create_clkout(self.cd_sys2x,     2 * sys_clk_freq)
        pll.create_clkout(self.cd_sys4x,     4 * sys_clk_freq)
        pll.create_clkout(self.cd_sys4x_dqs, 4 * sys_clk_freq, phase=90)
        pll.create_clkout(self.cd_idelay,    iodelay_clk_freq)

        self.idelayctrl = S7IDELAYCTRL(self.cd_idelay)

        # Video PLL.
        if with_video_pll:
            self.video_pll = video_pll = S7MMCM(speedgrade=-1)
            self.comb += video_pll.reset.eq(self.rst)
            video_pll.register_clkin(clk100, 100e6)
            video_pll.create_clkout(self.cd_hdmi,   40e6)
            video_pll.create_clkout(self.cd_hdmi5x, 5*40e6)


# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, *, sys_clk_freq=100e6, iodelay_clk_freq=200e6,
            with_ethernet          = False,
            with_etherbone         = False,
            eth_ip                 = "192.168.1.50",
            eth_reset_time         = "10e-3",
            eth_dynamic_ip         = False,
            with_hyperram          = False,
            with_sdcard            = False,
            with_jtagbone          = True,
            with_uartbone          = False,
            with_spi_flash         = False,
            with_led_chaser        = True,
            with_video_terminal    = False,
            with_video_framebuffer = False,
            **kwargs):
        platform = antmicro_datacenter_ddr4_test_board.Platform()

        # CRG --------------------------------------------------------------------------------------
        with_video_pll = (with_video_terminal or with_video_framebuffer)
        self.crg = _CRG(platform, sys_clk_freq, iodelay_clk_freq=iodelay_clk_freq, with_video_pll=with_video_pll)

        # SoCCore ----------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on data center test board", **kwargs)

        # DDR4 SDRAM RDIMM -------------------------------------------------------------------------
        if not self.integrated_main_ram_size:
            self.ddrphy = A7DDRPHY(platform.request("ddr4"),
                memtype         = "DDR4",
                iodelay_clk_freq = iodelay_clk_freq,
                sys_clk_freq     = sys_clk_freq,
                is_rdimm         = True,
            )
            self.add_sdram("sdram",
                phy                     = self.ddrphy,
                module                  = MTA18ASF2G72PZ(sys_clk_freq, "1:4"),
                l2_cache_size           = kwargs.get("l2_size", 8192),
                l2_cache_min_data_width = 256,
                size                    = 0x40000000,
            )

        # HyperRAM ---------------------------------------------------------------------------------
        if with_hyperram:
            self.hyperram = HyperRAM(platform.request("hyperram"), sys_clk_freq=sys_clk_freq)
            self.bus.add_slave("hyperram", slave=self.hyperram.bus, region=SoCRegion(origin=0x20000000, size=8*1024*1024))

        # SD Card ----------------------------------------------------------------------------------
        if with_sdcard:
            self.add_sdcard()

        # Ethernet / Etherbone ---------------------------------------------------------------------
        if with_ethernet or with_etherbone:
            # Traces between PHY and FPGA introduce ignorable delays of ~0.165ns +/- 0.015ns.
            # PHY chip does not introduce delays on TX (FPGA->PHY), however it includes 1.2ns
            # delay for RX CLK so we only need 0.8ns to match the desired 2ns.
            self.ethphy = LiteEthS7PHYRGMII(
                clock_pads = self.platform.request("eth_clocks"),
                pads       = self.platform.request("eth"),
                rx_delay   = 0.8e-9,
                hw_reset_cycles = math.ceil(float(eth_reset_time) * self.sys_clk_freq)
            )
            if with_ethernet:
                self.add_ethernet(phy=self.ethphy, dynamic_ip=eth_dynamic_ip)
            if with_etherbone:
                self.add_etherbone(phy=self.ethphy, ip_address=eth_ip)

        # UartBone ---------------------------------------------------------------------------------
        if with_uartbone:
            self.add_uartbone(baudrate=1e6)

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

        # Video ------------------------------------------------------------------------------------
        if with_video_terminal or with_video_framebuffer:
            self.videophy = VideoS7HDMIPHY(platform.request("hdmi_out"), clock_domain="hdmi")
            if with_video_terminal:
                self.add_video_terminal(phy=self.videophy, timings="800x600@60Hz", clock_domain="hdmi")
            if with_video_framebuffer:
                self.add_video_framebuffer(phy=self.videophy, timings="800x600@60Hz", clock_domain="hdmi")

        # SPI Flash --------------------------------------------------------------------------------
        if with_spi_flash:
            self.add_spi_flash(mode="4x", module=S25FL128S0(Codes.READ_1_1_4), with_master=True)

        # System I2C (behing multiplexer) ----------------------------------------------------------
        i2c_pads = platform.request('i2c')
        self.i2c = I2CMaster(i2c_pads)

    def generate_sdram_phy_py_header(self, output_file):
        os.makedirs(os.path.dirname(output_file), exist_ok=True)
        f = open(output_file, "w")
        f.write(get_sdram_phy_py_header(
            self.sdram.controller.settings.phy,
            self.sdram.controller.settings.timing))
        f.close()


# Build --------------------------------------------------------------------------------------------

class LiteDRAMSettingsEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, (ControllerSettings, GeomSettings, PhySettings, TimingSettings)):
            ignored = ["self", "refresh_cls"]
            return {k: v for k, v in vars(o).items() if k not in ignored}
        elif isinstance(o, Signal) and isinstance(o.reset, Constant):
            return o.reset
        elif isinstance(o, Constant):
            return o.value
        print('o', end=' = '); __import__('pprint').pprint(o)
        return super().default(o)

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=antmicro_datacenter_ddr4_test_board.Platform, description="LiteX SoC on DDR4 Datacenter Test Board.")
    parser.add_target_argument("--flash",            action="store_true",       help="Flash bitstream.")
    parser.add_target_argument("--sys-clk-freq",     default=100e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--iodelay-clk-freq", default=200e6, type=float, help="IODELAYCTRL frequency.")
    ethopts = parser.target_group.add_mutually_exclusive_group()
    ethopts.add_argument("--with-ethernet",                action="store_true",    help="Add Ethernet.")
    ethopts.add_argument("--with-etherbone",               action="store_true",    help="Add EtherBone.")
    parser.add_target_argument("--eth-ip",                 default="192.168.1.50", help="Ethernet/Etherbone IP address.")
    parser.add_target_argument("--eth-dynamic-ip",         action="store_true",    help="Enable dynamic Ethernet IP addresses setting.")
    parser.add_target_argument("--eth-reset-time",         default="10e-3",        help="Duration of Ethernet PHY reset.")
    parser.add_target_argument("--with-hyperram",          action="store_true",    help="Add HyperRAM.")
    parser.add_target_argument("--with-sdcard",            action="store_true",    help="Add SDCard.")
    parser.add_target_argument("--with-jtagbone",          action="store_true",    help="Add JTAGBone.")
    parser.add_target_argument("--with-uartbone",          action="store_true",    help="Add UartBone on 2nd serial.")
    parser.add_target_argument("--with-video-terminal",    action="store_true",    help="Enable Video Terminal (HDMI).")
    parser.add_target_argument("--with-video-framebuffer", action="store_true",    help="Enable Video Framebuffer (HDMI).")
    parser.add_target_argument("--with-spi-flash",         action="store_true",    help="Enable SPI Flash (MMAPed).")
    args = parser.parse_args()

    assert not (args.with_etherbone and args.eth_dynamic_ip)

    soc = BaseSoC(
        sys_clk_freq           = args.sys_clk_freq,
        iodelay_clk_freq       = args.iodelay_clk_freq,
        with_ethernet          = args.with_ethernet,
        with_etherbone         = args.with_etherbone,
        eth_ip                 = args.eth_ip,
        eth_dynamic_ip         = args.eth_dynamic_ip,
        with_hyperram          = args.with_hyperram,
        with_sdcard            = args.with_sdcard,
        with_jtagbone          = args.with_jtagbone,
        with_uartbone          = args.with_uartbone,
        with_spi_flash         = args.with_spi_flash,
        with_video_terminal    = args.with_video_terminal,
        with_video_framebuffer = args.with_video_framebuffer,
        **parser.soc_argdict)
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)
        builder.soc.generate_sdram_phy_py_header(os.path.join(builder.output_dir, "sdram_init.py"))
        # LiteDRAM settings (controller, phy, geom, timing)
        with open(os.path.join(builder.output_dir, 'litedram_settings.json'), 'w') as f:
            json.dump(builder.soc.sdram.controller.settings, f, cls=LiteDRAMSettingsEncoder, indent=4)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

    if args.flash:
        prog = soc.platform.create_programmer()
        prog.flash(0, builder.get_bitstream_filename(mode="flash"))

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

# Board diagram/pinout:
# http://www.fabienm.eu/flf/wp-content/uploads/2021/08/Tang-Nano-4K-specifications.jpg
# http://www.fabienm.eu/flf/wp-content/uploads/2021/08/Tang-Nano-4K-GW1NSR-4C-FPGA-board-pinout-diagram.jpg


from migen import *

from litex.build.generic_platform import *
from litex.build.gowin.platform import GowinPlatform
from litex.build.openfpgaloader import OpenFPGALoader

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk27",  0, Pins("45"), IOStandard("LVCMOS33")),

    # Leds
    ("user_led", 0, Pins("10"), IOStandard("LVCMOS33")),

    # Buttons.
    ("user_btn", 0, Pins("14"), IOStandard("LVCMOS18")),
    ("user_btn", 1, Pins("15"), IOStandard("LVCMOS18")),

    # Serial (FIXME: For tests, change or remove.)
    ("serial", 0,
        Subsignal("rx", Pins("44")), # CAM_SCL
        Subsignal("tx", Pins("46")), # CAM_SDA
        IOStandard("LVCMOS33")
    ),

    # SPIFlash
    ("spiflash", 0,
        Subsignal("cs_n", Pins("2"),  IOStandard("LVCMOS33")),
        Subsignal("clk",  Pins("1"),  IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("47"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("48"), IOStandard("LVCMOS33")),
        Subsignal("wp",   Pins("8"),  IOStandard("LVCMOS33")),
        Subsignal("hold", Pins("9"),  IOStandard("LVCMOS33")),
    ),
    ("spiflash4x", 0,
        Subsignal("cs_n", Pins("2")),
        Subsignal("clk",  Pins("1")),
        Subsignal("dq",   Pins("48 47 8 9")),
        IOStandard("LVCMOS33")
    ),

    # HyperRAM (embedded in SIP, requires specific IO naming).
    ("O_hpram_ck",      0, Pins(1)),
    ("O_hpram_ck_n",    0, Pins(1)),
    ("O_hpram_cs_n",    0, Pins(1)),
    ("O_hpram_reset_n", 0, Pins(1)),
    ("IO_hpram_dq",     0, Pins(8)),
    ("IO_hpram_rwds",   0, Pins(1)),

    # HDMI.
    ("hdmi", 0,
        Subsignal("clk_p",   Pins("28")),
        Subsignal("clk_n",   Pins("27")),
        Subsignal("data0_p", Pins("30")),
        Subsignal("data0_n", Pins("29")),
        Subsignal("data1_p", Pins("32")),
        Subsignal("data1_n", Pins("31")),
        Subsignal("data2_p", Pins("35")),
        Subsignal("data2_n", Pins("34")),
        Misc("PULL_MODE=NONE"),
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ["P6", "30 29 28 27 23 22 21 20 19 18 17 13 16  9  8 33  2  -  -  -  - 15"],
    ["P7", "31 32 34 35 10 39 40 41 42 43 47 48  1 46 44  -  -  6  3  4  7 14"],
]

# Platform -----------------------------------------------------------------------------------------

class Platform(GowinPlatform):
    default_clk_name   = "clk27"
    default_clk_period = 1e9/27e6

    def __init__(self, toolchain="gowin"):
        # Educational GoWin IDE v. 1.9.8.03
        GowinPlatform.__init__(self, "GW1NSR-LV4CQN48PC6/I5", _io, _connectors, toolchain=toolchain, devicename="GW1NSR-4C")
        # Licensed GoWin IDE v. 1.9.7.06 Beta
        # GowinPlatform.__init__(self, "GW1NSR-LV4CQN48PC7/I6", _io, _connectors, toolchain=toolchain, devicename="GW1NSR-4C")
        self.toolchain.options["use_mode_as_gpio"] = 1
        self.toolchain.options["use_mspi_as_gpio"] = 1
        self.toolchain.options["use_done_as_gpio"] = 1

    def create_programmer(self):
        return OpenFPGALoader("tangnano4k")

    def do_finalize(self, fragment):
        GowinPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk27", loose=True), 1e9/27e6)



from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk100", 0, Pins("C18"), IOStandard("LVCMOS33")),

    # Leds
    ("user_led", 0, Pins("T20"),  IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("U20"),  IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("W20"),  IOStandard("LVCMOS33")),

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("R19")),
        Subsignal("rx", Pins("P19")),
        IOStandard("LVCMOS33"),
    ),
    ("serial", 1,
        Subsignal("tx", Pins("U21")),
        Subsignal("rx", Pins("T21")),
        IOStandard("LVCMOS33"),
    ),

    # DDR3 SDRAM (voltage changed from 1.5V to 1.35V)
    ("ddram", 0,
        Subsignal("a", Pins(
            "M2 M5 M3 M1 L6 P1 N3 N2",
            "M6 R1 L5 N5 N4 P2 P6"),
            IOStandard("SSTL135")),
        Subsignal("ba", Pins("L3 K6 L4"), IOStandard("SSTL135")),
        Subsignal("ras_n", Pins("J4"), IOStandard("SSTL135")),
        Subsignal("cas_n", Pins("K3"), IOStandard("SSTL135")),
        Subsignal("we_n", Pins("L1"), IOStandard("SSTL135")),
        Subsignal("dm", Pins("G3 F1"), IOStandard("SSTL135")),
        Subsignal("dq", Pins(
            "G2 H4 H5 J1 K1 H3 H2 J5",
            "E3 B2 F3 D2 C2 A1 E2 B1"),
            IOStandard("SSTL135"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("dqs_p", Pins("K2 E1"),
            IOStandard("DIFF_SSTL135"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("dqs_n", Pins("J2 D1"),
            IOStandard("DIFF_SSTL135"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("clk_p", Pins("P5"), IOStandard("DIFF_SSTL135")),
        Subsignal("clk_n", Pins("P4"), IOStandard("DIFF_SSTL135")),
        Subsignal("cke", Pins("J6"), IOStandard("SSTL135")),
        Subsignal("odt", Pins("K4"), IOStandard("SSTL135")),
        Subsignal("reset_n", Pins("G1"), IOStandard("SSTL135")),
        Misc("SLEW=FAST"),
    ),

    # eMMC - there are pullups on the board so we don't enable them here
    ("sdcard", 0,
        Subsignal("data", Pins("P17 W17 R18 V18")),
        Subsignal("cmd",  Pins("Y19")),
        Subsignal("clk",  Pins("Y18")),
            #Subsignal("cd",   Pins(),
        Misc("SLEW=FAST"),
        IOStandard("LVCMOS33"),
    ),
    # RGMII Ethernet
    ("eth_ref_clk", 0, Pins("H19"), IOStandard("LVCMOS33")),  # 125 MHz if enabled?
    ("eth_clocks", 0,
        Subsignal("tx", Pins("J19")),
        Subsignal("rx", Pins("K19")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 0,
        Subsignal("rst_n",   Pins("N18"), IOStandard("LVCMOS33")),
        Subsignal("int_n",   Pins("N20"), IOStandard("LVCMOS33")),
        Subsignal("mdio",    Pins("M21"), IOStandard("LVCMOS33")),
        Subsignal("mdc",     Pins("N22"), IOStandard("LVCMOS33")),
        Subsignal("rx_ctl",  Pins("M22"), IOStandard("LVCMOS33")),
        Subsignal("rx_data", Pins("L20 L21 K21 K22"), IOStandard("LVCMOS33")),
        Subsignal("tx_ctl",  Pins("J22"), IOStandard("LVCMOS33")),
        Subsignal("tx_data", Pins("G20 H20 H22 J21"), IOStandard("LVCMOS33")),
    ),

    # PCIe
    ("pcie_x1", 0,
        # Subsignal("rst_n", Pins(""), IOStandard("LVCMOS33")),
        Subsignal("clk_p", Pins("F6")),
        Subsignal("clk_n", Pins("E6")),
        Subsignal("rx_p",  Pins("B8")),
        Subsignal("rx_n",  Pins("A8")),
        Subsignal("tx_p",  Pins("B4")),
        Subsignal("tx_n",  Pins("A4"))
    ),

    # USB ULPI
    ("ulpi_clock", 0, Pins("W19"), IOStandard("LVCMOS33")),
    ("ulpi", 0,
        Subsignal("data", Pins("AB18 AA18 AA19 AB20 AA20 AB21 AA21 AB22")),
        Subsignal("dir", Pins("W21")),
        Subsignal("stp", Pins("Y22")),
        Subsignal("nxt", Pins("W22")),
        Subsignal("rst", Pins("V20")),
        IOStandard("LVCMOS33"), Misc("SLEW=FAST")
    ),

    ("ulpi_clock", 1, Pins("V4"), IOStandard("LVCMOS33")),
    ("ulpi", 1,
        Subsignal("data", Pins("AB2 AA3 AB3 Y4 AA4 AB5 AA5 AB6")),
        Subsignal("dir", Pins("AB7")),
        Subsignal("stp", Pins("AA6")),
        Subsignal("nxt", Pins("AB8")),
        Subsignal("rst", Pins("AA8")),
        IOStandard("LVCMOS33"), Misc("SLEW=FAST")
    ),

]

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk100"
    default_clk_period = 1e9/100e6

    def __init__(self, device="xc7a100tfgg484-1", toolchain="vivado"):
        Xilinx7SeriesPlatform.__init__(self, device, _io, toolchain=toolchain)
        # self.toolchain.bitstream_commands = \
        #     ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]"]
        # self.toolchain.additional_commands = \
        #     ["write_cfgmem -force -format bin -interface spix4 -size 16 "
        #      "-loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]
        self.add_platform_command("set_property INTERNAL_VREF 0.675 [get_iobanks 35]")

    def create_programmer(self):
        bscan_spi = "bscan_spi_xc7a100t.bit" if "xc7a100t" in self.device else "bscan_spi_xc7a35t.bit"
        return OpenOCD("openocd_xc7_ft2232.cfg", bscan_spi)

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk100", loose=True), 1e9/100e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2014-2019 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.altera import AlteraPlatform
from litex.build.altera.programmer import USBBlaster

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk
    ("clk50", 0, Pins("R8"), IOStandard("3.3-V LVTTL")),

    # Leds
    ("user_led", 0, Pins("A15"), IOStandard("3.3-V LVTTL")),
    ("user_led", 1, Pins("A13"), IOStandard("3.3-V LVTTL")),
    ("user_led", 2, Pins("B13"), IOStandard("3.3-V LVTTL")),
    ("user_led", 3, Pins("A11"), IOStandard("3.3-V LVTTL")),
    ("user_led", 4, Pins("D1"),  IOStandard("3.3-V LVTTL")),
    ("user_led", 5, Pins("F3"),  IOStandard("3.3-V LVTTL")),
    ("user_led", 6, Pins("B1"),  IOStandard("3.3-V LVTTL")),
    ("user_led", 7, Pins("L3"),  IOStandard("3.3-V LVTTL")),

    # Button
    ("key", 0, Pins("J15"), IOStandard("3.3-V LVTTL")),
    ("key", 1, Pins("E1"),  IOStandard("3.3-V LVTTL")),

    # Switches
    ("sw", 0, Pins("M1"),  IOStandard("3.3-V LVTTL")),
    ("sw", 1, Pins("T8"),  IOStandard("3.3-V LVTTL")),
    ("sw", 2, Pins("B9"),  IOStandard("3.3-V LVTTL")),
    ("sw", 3, Pins("M15"), IOStandard("3.3-V LVTTL")),

    # Serial
    ("serial", 0,
        # Compatible with cheap FT232 based cables (ex: Gaoominy 6Pin Ftdi Ft232Rl Ft232)
        # GND on JP1 Pin 12.
        Subsignal("tx", Pins("JP1:10"), IOStandard("3.3-V LVTTL")),
        Subsignal("rx", Pins("JP1:8"),  IOStandard("3.3-V LVTTL"))
    ),

    # SDR SDRAM
    ("sdram_clock", 0, Pins("R4"), IOStandard("3.3-V LVTTL")),
    ("sdram", 0,
        Subsignal("a",     Pins(
            "P2 N5 N6 M8 P8 T7 N8 T6",
            "R1 P1 N2 N1 L4")),
        Subsignal("ba",    Pins("M7 M6")),
        Subsignal("cs_n",  Pins("P6")),
        Subsignal("cke",   Pins("L7")),
        Subsignal("ras_n", Pins("L2")),
        Subsignal("cas_n", Pins("L1")),
        Subsignal("we_n",  Pins("C2")),
        Subsignal("dq", Pins(
            "G2 G1 L8 K5 K2 J2 J1 R7",
            "T4 T2 T3 R3 R5 P3 N3 K1")),
        Subsignal("dm", Pins("R6 T5")),
        IOStandard("3.3-V LVTTL")
    ),

    # ECPS
    ("epcs", 0,
        Subsignal("data0", Pins("H2")),
        Subsignal("dclk",  Pins("H1")),
        Subsignal("ncs0",  Pins("D2")),
        Subsignal("asd0",  Pins("C1")),
        IOStandard("3.3-V LVTTL")
    ),

    # I2C
    ("i2c", 0,
        Subsignal("sclk", Pins("F2")),
        Subsignal("sdat", Pins("F1")),
        IOStandard("3.3-V LVTTL")
    ),

    # Accelerometer
    ("acc", 0,
        Subsignal("cs_n", Pins("G5")),
        Subsignal("int",  Pins("M2")),
        IOStandard("3.3-V LVTTL")
    ),

    # ADC
    ("adc", 0,
        Subsignal("cs_n",  Pins("A10")),
        Subsignal("saddr", Pins("B10")),
        Subsignal("sclk",  Pins("B14")),
        Subsignal("sdat",  Pins("A9")),
        IOStandard("3.3-V LVTTL")
    ),

    # GPIOs
    ("gpio_0", 0, Pins(
        "D3 C3  A2  A3  B3  B4  A4  B5",
        "A5 D5  B6  A6  B7  D6  A7  C6",
        "C8 E6  E7  D8  E8  F8  F9  E9",
        "C9 D9 E11 E10 C11 B11 A12 D11",
        "D12 B12"),
        IOStandard("3.3-V LVTTL")
    ),
    ("gpio_1", 0, Pins(
        "F13 T15 T14 T13 R13 T12 R12 T11",
        "T10 R11 P11 R10 N12  P9  N9 N11",
        "L16 K16 R16 L15 P15 P16 R14 N16",
        "N15 P14 L14 N14 M10 L13 J16 K15",
        "J13 J14"),
        IOStandard("3.3-V LVTTL")
    ),
    ("gpio_2", 0, Pins(
        "A14 B16 C14 C16 C15 D16 D15 D14",
        "F15 F16 F14 G16 G15"),
        IOStandard("3.3-V LVTTL")
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    # PIN    0  1  2   3   4   5   6   7   8   9   10  11  12  13  14  15  16  17  18 19 20  21  22  23  24  25  26  27  28 29 30 31  32  33  34  35  36  37  38  39  40
    ("JP1", "-  A8 D3  B8  C3  A2  A3  B3  B4  A4  B5  -   -   A5  D5  B6  A6  B7  D6 A7 C6  C8  E6  E7  D8  E8  F8  F9  E9  - -  C9  D9  E11 E10 C11 B11 A12 D11 D12 B12"),
    ("JP2", "-  T9 F13 R9  T15 T14 T13 R13 T12 R12 T11 -   -   T10 R11 P11 R10 N12 P9 N9 N11 L16 K16 R16 L15 P15 P16 R14 N16 - -  N15 P14 L14 N14 M10 L13 J16 K15 J13 J14"),
    ("JP3", "-  -  E15 E16 M16 A14 B16 C14 C16 C15 D16 D15 D14 F15 F16 F14 G16 G15 -  -  -   -   -   -   -   -   -")
]

# Platform -----------------------------------------------------------------------------------------

class Platform(AlteraPlatform):
    default_clk_name   = "clk50"
    default_clk_period = 1e9/50e6

    def __init__(self, toolchain="quartus"):
        AlteraPlatform.__init__(self, "EP4CE22F17C6", _io, _connectors, toolchain=toolchain)

    def create_programmer(self):
        return USBBlaster()

    def do_finalize(self, fragment):
        AlteraPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk50", loose=True), 1e9/50e6)



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Derek Mulcahy <derekmulcahy@gmail.com>
# Copyright (c) 2019-2020 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

import os

from migen import *

from litex.gen import *

from litex_boards.platforms import krtkl_snickerdoodle

from litex.soc.interconnect import axi
from litex.soc.interconnect import wishbone

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser

# UTILS ---------------------------------------------------------------------------------------------

def load_ps7(soc, xci_file):
    odir = os.path.join("build", "krtkl_snickerdoodle", "gateware", "xci")
    os.makedirs(odir, exist_ok=True)
    file = "snickerdoodle_ps7.xci"
    dst = os.path.join(odir, file)
    if xci_file is None:
        src = "https://technicaltoys-support.s3.amazonaws.com/xci/" + file
        os.system("wget " + src + " -O " + dst)
    else:
        os.system("cp -p  " + xci_file + " " + dst)
    soc.cpu.set_ps7_xci(dst)

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq, use_ps7_clk=False):
        self.rst    = Signal()
        self.cd_sys = ClockDomain()

        # # #

        if use_ps7_clk:
            assert sys_clk_freq == 100e6
            self.comb += ClockSignal("sys").eq(ClockSignal("ps7"))
            self.comb += ResetSignal("sys").eq(ResetSignal("ps7") | self.rst)
        else:
            self.pll = pll = S7MMCM(speedgrade=-1)
            self.comb += pll.reset.eq(self.rst)
            pll.register_clkin(platform.request(platform.default_clk_name),
                               platform.default_clk_freq)
            pll.create_clkout(self.cd_sys, sys_clk_freq)
            # Ignore sys_clk to pll.clkin path created by SoC's rst.
            platform.add_false_path_constraints(self.cd_sys.clk, pll.clkin)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):

    def __init__(self, variant="z7-10", sys_clk_freq=100e6,
        with_led_chaser = True,
        ext_clk_freq    = None,
        xci_file        = None,
        **kwargs):
        platform = krtkl_snickerdoodle.Platform(variant=variant)

        # CRG --------------------------------------------------------------------------------------
        if ext_clk_freq:
            platform.default_clk_freq   = ext_clk_freq
            platform.default_clk_period = 1e9 / ext_clk_freq
        use_ps7_clk = (kwargs.get("cpu_type", None) == "zynq7000")
        self.crg = _CRG(platform, sys_clk_freq, use_ps7_clk)

        # SoCCore ----------------------------------------------------------------------------------
        if kwargs.get("cpu_type", None) == "zynq7000":
            kwargs["integrated_sram_size"] = 0
            kwargs["with_uart"]            = False
            self.mem_map = {"csr": 0x4000_0000}  # Zynq GP0 default
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on Snickerdoodle", **kwargs)

        # Zynq7000 Integration ---------------------------------------------------------------------
        if kwargs.get("cpu_type", None) == "zynq7000":
            load_ps7(self, xci_file)

            # Connect AXI GP0 to the SoC with base address of 0x43c00000 (default one)
            wb_gp0  = wishbone.Interface()
            self.submodules += axi.AXI2Wishbone(
                axi          = self.cpu.add_axi_gp_master(),
                wishbone     = wb_gp0,
                base_address = self.mem_map["csr"])
            self.bus.add_master(master=wb_gp0)

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=krtkl_snickerdoodle.Platform, description="LiteX SoC on Snickerdoodle.")
    parser.add_target_argument("--variant",      default="z7-10",           help="Board variant (z7-10 or z7-20).")
    parser.add_target_argument("--ext-clk-freq", default=10e6,  type=float, help="External Clock Frequency.")
    parser.add_target_argument("--sys-clk-freq", default=100e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--xci-file",     help="XCI file for PS7 configuration.")
    parser.add_target_argument("--target",       help="Vivado programmer target.")
    args = parser.parse_args()

    soc = BaseSoC(
        variant      = args.variant,
        sys_clk_freq = args.sys_clk_freq,
        ext_clk_freq = args.ext_clk_freq,
        xci_file     = args.xci_file,
        **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"), target=args.target, device=1)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2019 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *
from migen.genlib.resetsync import AsyncResetSynchronizer

from litex.gen import *

from litex_boards.platforms import camlink_4k

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser

from litedram.modules import MT41K64M16
from litedram.phy import ECP5DDRPHY

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq):
        self.rst        = Signal()
        self.cd_init    = ClockDomain()
        self.cd_por     = ClockDomain()
        self.cd_sys     = ClockDomain()
        self.cd_sys2x   = ClockDomain()
        self.cd_sys2x_i = ClockDomain()

        # # #

        self.stop = Signal()

        # clk / rst
        clk27 = platform.request("clk27")

        # power on reset
        por_count = Signal(16, reset=2**16-1)
        por_done  = Signal()
        self.comb += self.cd_por.clk.eq(ClockSignal())
        self.comb += por_done.eq(por_count == 0)
        self.sync.por += If(~por_done, por_count.eq(por_count - 1))

        # pll
        self.pll = pll = ECP5PLL()
        self.comb += pll.reset.eq(~por_done | self.rst)
        pll.register_clkin(clk27, 27e6)
        pll.create_clkout(self.cd_sys2x_i, 2*sys_clk_freq)
        pll.create_clkout(self.cd_init, 27e6)
        self.specials += [
            Instance("ECLKSYNCB",
                i_ECLKI = self.cd_sys2x_i.clk,
                i_STOP  = self.stop,
                o_ECLKO = self.cd_sys2x.clk),
            Instance("CLKDIVF",
                p_DIV     = "2.0",
                i_ALIGNWD = 0,
                i_CLKI    = self.cd_sys2x.clk,
                i_RST     = self.cd_sys2x.rst,
                o_CDIVX   = self.cd_sys.clk),
            AsyncResetSynchronizer(self.cd_sys, ~pll.locked)
        ]

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=81e6, toolchain="trellis", with_led_chaser=True, **kwargs):
        platform = camlink_4k.Platform(toolchain=toolchain)


        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq)

        # SoCCore ----------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on Cam Link 4K", **kwargs)

        # DDR3 SDRAM -------------------------------------------------------------------------------
        if not self.integrated_main_ram_size:
            self.ddrphy = ECP5DDRPHY(
                platform.request("ddram"),
                sys_clk_freq=sys_clk_freq)
            self.comb += self.crg.stop.eq(self.ddrphy.init.stop)
            self.add_sdram("sdram",
                phy           = self.ddrphy,
                module        = MT41K64M16(sys_clk_freq, "1:2"),
                l2_cache_size = kwargs.get("l2_size", 8192)
            )

        # Leds -------------------------------------------------------------------------------------
        # Disable leds when serial is used.
        if platform.lookup_request("serial", loose=True) is None and with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=camlink_4k.Platform, description="LiteX SoC on Cam Link 4K.")
    parser.add_target_argument("--sys-clk-freq", default=81e6, type=float, help="System clock frequency.")
    args = parser.parse_args()

    soc = BaseSoC(
        sys_clk_freq = args.sys_clk_freq,
        toolchain    = args.toolchain,
        **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram", ext=".svf")) # FIXME

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Hans Baier <hansfbaier@gmail.com>
#
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.altera            import AlteraPlatform
from litex.build.altera.programmer import USBBlaster
from litex.build.generic_platform  import Pins, IOStandard, Subsignal, Misc

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk.
    ("clk50", 0, Pins("AF14"), IOStandard("3.3-V LVTTL")),

    # Leds.
    ("user_led", 0, Pins("AF10"), IOStandard("3.3-V LVTTL")),
    ("user_led", 1, Pins("AD10"), IOStandard("3.3-V LVTTL")),
    ("user_led", 2, Pins("AE11"), IOStandard("3.3-V LVTTL")),
    ("user_led", 3, Pins("AD7"),  IOStandard("3.3-V LVTTL")),

    # Buttons.
    ("user_btn", 0, Pins("AE9"),  IOStandard("3.3-V LVTTL")),
    ("user_btn", 1, Pins("AE12"), IOStandard("3.3-V LVTTL")),
    ("user_btn", 2, Pins("AD9"),  IOStandard("3.3-V LVTTL")),
    ("user_btn", 3, Pins("AD11"), IOStandard("3.3-V LVTTL")),

    # Switches
    ("user_sw", 0, Pins("W25"),  IOStandard("3.3-V LVTTL")),
    ("user_sw", 1, Pins("V25"),  IOStandard("3.3-V LVTTL")),
    ("user_sw", 2, Pins("AC28"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 3, Pins("AC29"), IOStandard("3.3-V LVTTL")),

    # MiSTer SDRAM (via GPIO expansion board on J2).
    ("sdram_clock", 0, Pins("D10"), IOStandard("3.3-V LVTTL")),
    ("sdram", 0,
        Subsignal("a",     Pins(
            "B1 C2 B2 D2 D9 C7 E12 B7",
            "D12 A11 B6 D11 A10")),
        Subsignal("ba",    Pins("B5 A4")),
        Subsignal("cs_n",  Pins("A3")),
        Subsignal("cke",   Pins("B3")), # CKE not connected on XS 2.2/2.4.
        Subsignal("ras_n", Pins("E9")),
        Subsignal("cas_n", Pins("A6")),
        Subsignal("we_n",  Pins("A5")),
        Subsignal("dq", Pins(
            "F14 G15 F15 H15 G13 A13 H14 B13",
            "C13 C8 B12 B8 F13 C12 B11 E13"),
        ),
        Subsignal("dm", Pins("AB27 AA26")), # DQML/DQMH not connected on XS 2.2/2.4
        IOStandard("3.3-V LVTTL"),
        Misc("CURRENT_STRENGTH_NEW \"MAXIMUM CURRENT\""),
    ),

    # DDR3 SDRAM.
    ("ddram", 0,
        Subsignal("a", Pins(
            "AJ14 AK14 AH12 AJ12 AG15 AH15 AK12 AK13",
            "AH13 AH14 AJ9  AK9  AK7  AK8  AG12"),
            IOStandard("SSTL15"),
        ),
        Subsignal("ba",    Pins("AH10 AJ11 AK11"), IOStandard("SSTL-15 CLASS I"), ),
        Subsignal("ras_n", Pins("AH8"), IOStandard("SSTL-15 CLASS I")),
        Subsignal("cas_n", Pins("AH7"), IOStandard("SSTL-15 CLASS I")),
        Subsignal("we_n",  Pins("AJ6"), IOStandard("SSTL-15 CLASS I")),
        Subsignal("dm", Pins("AH17 AG23 AK23 AJ27"),
            IOStandard("SSTL-15 CLASS I"),
            Misc("OUTPUT_TERMINATION=SERIES 50 OHM WITH CALIBRATION")
        ),
        Subsignal("dq", Pins(
            "AF18 AE17 AG16 AF16 AH20 AG21 AJ16 AH18",
            "AK18 AJ17 AG18 AK19 AG20 AF19 AJ20 AH24",
            "AE19 AE18 AG22 AK22 AF21 AF20 AH23 AK24",
            "AF24 AF23 AJ24 AK26 AE23 AE22 AG25 AK27"),
            IOStandard("SSTL-15 CLASS I"),
            Misc("INPUT_TERMINATION=PARALLEL 50 OHM WITH CALIBRATION"),
            Misc("OUTPUT_TERMINATION=SERIES 50 OHM WITH CALIBRATION"),
        ),
        Subsignal("dqs_p", Pins("V16 V17 Y17 AC20"),
            IOStandard("DIFFERENTIAL 1.5-V SSTL CLASS I"),
            Misc("INPUT_TERMINATION=PARALLEL 50 OHM WITH CALIBRATION"),
            Misc("OUTPUT_TERMINATION=SERIES 50 OHM WITH CALIBRATION")
        ),
        Subsignal("dqs_n", Pins("W16 W17 AA18 AD19"),
            IOStandard("DIFFERENTIAL 1.5-V SSTL CLASS I"),
            Misc("INPUT_TERMINATION=PARALLEL 50 OHM WITH CALIBRATION"),
            Misc("OUTPUT_TERMINATION=SERIES 50 OHM WITH CALIBRATION")
        ),
        Subsignal("clk_p", Pins("AA14"),
            IOStandard("DIFFERENTIAL 1.5-V SSTL CLASS I"),
            Misc("OUTPUT_TERMINATION=SERIES 50 OHM WITH CALIBRATION"),
            Misc("D5_DELAY=2")
        ),
        Subsignal("clk_n", Pins("AA15"),
            IOStandard("DIFFERENTIAL 1.5-V SSTL CLASS I"),
            Misc("OUTPUT_TERMINATION=SERIES 50 OHM WITH CALIBRATION"),
            Misc("D5_DELAY=2")
        ),
        Subsignal("cs_n",    Pins("AB15"), IOStandard("SSTL-15 CLASS I")),
        Subsignal("cke",     Pins("AJ21"), IOStandard("SSTL-15 CLASS I")),
        Subsignal("odt",     Pins("AE16"), IOStandard("SSTL-15 CLASS I")),
        Subsignal("reset_n", Pins("AK21"), IOStandard("SSTL-15 CLASS I")),
        Subsignal("rzq",     Pins("AG1"),  IOStandard("SSTL-15")),
        Misc("CURRENT_STRENGTH_NEW=MAXIMUM CURRENT")
        ),

    # VGA.
    ("vga", 0,
        Subsignal("sync_n",  Pins("AG2")),
        Subsignal("blank_n", Pins("AH3")),
        Subsignal("clk",     Pins("W20")),
        Subsignal("hsync_n", Pins("AD12")),
        Subsignal("vsync_n", Pins("AC12")),
        Subsignal("r", Pins("AG5  AA12 AB12 AF6  AG6  AJ2  AH5  AJ1")),
        Subsignal("g", Pins("Y21  AA25 AB26 AB22 AB23 AA24 AB25 AE27")),
        Subsignal("b", Pins("AE28 Y23  Y24  AG28 AF28 V23  W24  AF29")),
        IOStandard("3.3-V LVTTL")
    ),

    # IrDA.
    ("irda", 0,
        Subsignal("irda_rxd", Pins("AH2")),
        IOStandard("3.3-V LVTTL")
    ),

    # Temperatue.
    ("temperature", 0,
        Subsignal("temp_cs_n", Pins("AF8")),
        Subsignal("temp_din",  Pins("AG7")),
        Subsignal("temp_dout", Pins("AG1")),
        Subsignal("temp_sclk", Pins("AF9")),
        IOStandard("3.3-V LVTTL")
    ),

    # Audio.
    ("audio", 0,
        Subsignal("aud_adclrck",  Pins("AG30")),
        Subsignal("aud_adcdat",   Pins("AC27")),
        Subsignal("aud_daclrck",  Pins("AH4")),
        Subsignal("aud_dacdat",   Pins("AG3")),
        Subsignal("aud_xck",      Pins("AC9")),
        Subsignal("aud_bclk",     Pins("AE7")),
        Subsignal("aud_i2c_sclk", Pins("AH30")),
        Subsignal("aud_i2c_sdat", Pins("AF30")),
        Subsignal("aud_mute",     Pins("AD26")),
        IOStandard("3.3-V LVTTL")
    ),

    # GPIO Serial.
    ("gpio_serial", 0,
        Subsignal("tx", Pins("J3:9")),
        Subsignal("rx", Pins("J3:10")),
        IOStandard("3.3-V LVTTL"))
]

# Connectors ---------------------------------------------------------------------------------------

_connectors_hsmc_gpio_daughterboard = [
    ("J2", "- G15 F14 H15 F15 A13 G13 B13 H14 B11 E13 - - " +
           "C12 F13 B8 B12 C8 C13 A10 D10 A11 D11 B7 D12 C7 E12 A5 D9 - - " +
           "A6 E9 A3 B5 A4 B6 B1 C2 B2 D2"),
    ("J2p", "- D1 E1 E11 F11"), # Top to bottom, starting with 57.

    ("J3", "- AB27 F8 AA26 F9 B3 G8 C3 H8 D4 H7 - - " +
           "E4 J7 E2 K8 E3 K7 E6 J9 E7 J10 C4 J12 D5 G10 C5 J12 - - " +
           "D6 K12 F6 G11 G7 G12 D7 A8 E8 A9"),
    ("J3p", "- C9 C10 H12 H13"), # Top to bottom, starting with 117.

    ("J4", "- - - AD3 AE1 AD4 AE2 - - AB3 AC1 - - " +
           "AB4 AC2 - - Y3 AA1 Y4 AA2 - - V3 W1 V4 W2 - - - -" +
           "T3 U1 T4 R1 - R2 P3 U2 P4 -"),
    ("J4p", "- M3 M4 - H3 H4 J14 AD29 - N1 N2 - J1 J2") # Top to bottom, starting with 169.
]

# Platform -----------------------------------------------------------------------------------------

_device_map = {
    "revb" : "5CSXFC6D6F31C8ES",
    "revc" : "5CSXFC6D6F31C8ES",
    "revd" : "5CSXFC6D6F31C8",
}

class Platform(AlteraPlatform):
    default_clk_name   = "clk50"
    default_clk_period = 1e9/50e6

    def __init__(self, revision="revd", toolchain="quartus"):
        assert revision in _device_map.keys()
        self.revision = revision
        AlteraPlatform.__init__(self, _device_map[revision], _io, connectors=_connectors_hsmc_gpio_daughterboard, toolchain=toolchain)

    def create_programmer(self):
        return USBBlaster(cable_name="CV SoCKit")

    def do_finalize(self, fragment):
        AlteraPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk50", loose=True), 1e9/50e6)



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Michael Welling <mwelling@ieee.org>
# Copyright (c) 2020 Sean Cross <sean@xobs.io>
# Copyright (c) 2020 Drew Fustini <drew@pdp7.com>
# Copyright (c) 2020 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *
from migen.genlib.resetsync import AsyncResetSynchronizer

from litex.gen import *

from litex.build.io import DDROutput

from litex_boards.platforms import hackaday_hadbadge

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *

from litedram import modules as litedram_modules
from litedram.phy import GENSDRPHY
from litedram.modules import AS4C32M8

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq):
        self.rst       = Signal()
        self.cd_sys    = ClockDomain()
        self.cd_sys_ps = ClockDomain()

        # # #

        # Clk / Rst
        clk8  = platform.request("clk8")

        # PLL
        self.pll = pll = ECP5PLL()
        pll.pfd_freq_range = (8e6, 400e6) # Lower Min from 10MHz to 8MHz.
        self.comb += pll.reset.eq(self.rst)
        pll.register_clkin(clk8, 8e6)
        pll.create_clkout(self.cd_sys,    sys_clk_freq)
        pll.create_clkout(self.cd_sys_ps, sys_clk_freq, phase=90)

        # SDRAM clock
        self.specials += DDROutput(1, 0, platform.request("sdram_clock"), ClockSignal("sys_ps"))

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, toolchain="trellis", sys_clk_freq=48e6, sdram_module_cls="AS4C32M8", **kwargs):
        platform = hackaday_hadbadge.Platform(toolchain=toolchain)

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq)

        # SoCCore ---------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on Hackaday Badge", **kwargs)

        # SDR SDRAM --------------------------------------------------------------------------------
        if not self.integrated_main_ram_size:
            self.sdrphy = GENSDRPHY(platform.request("sdram"), sys_clk_freq)
            self.add_sdram("sdram",
                phy           = self.sdrphy,
                module        = AS4C32M8(sys_clk_freq, "1:1"),
                l2_cache_size = kwargs.get("l2_size", 8192)
            )

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=hackaday_hadbadge.Platform, description="LiteX SoC on Hackaday Badge.")
    parser.add_target_argument("--sys-clk-freq", default=48e6, type=float, help="System clock frequency.")
    args = parser.parse_args()

    soc = BaseSoC(
        toolchain    = args.toolchain,
        sys_clk_freq = args.sys_clk_freq,
        **parser.soc_argdict)
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Shinken Sanada <sanadashinken@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *

from litex.gen import *

from litex_boards.platforms import ego1

from litex.soc.cores.clock import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser
from litex.soc.cores.video import VideoVGAPHY

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq):
        self.rst    = Signal()
        self.cd_sys = ClockDomain()
        self.cd_vga = ClockDomain()

        self.pll = pll = S7PLL(speedgrade=-1)
        self.comb += pll.reset.eq(~platform.request("cpu_reset") | self.rst)
        pll.register_clkin(platform.request("clk100"), 100e6)
        pll.create_clkout(self.cd_sys,       sys_clk_freq)
        pll.create_clkout(self.cd_vga,       25e6)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=100e6, with_led_chaser=True, with_video_terminal=False, **kwargs):
        platform = ego1.Platform()

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq)

        # SoCCore ----------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on EGO1 Board", **kwargs)

        # VGA terminal -----------------------------------------------------------------------------
        if with_video_terminal:
            self.videophy = VideoVGAPHY(platform.request("vga"), clock_domain="vga")
            self.add_video_terminal(phy=self.videophy, timings="640x480@75Hz", clock_domain="vga")

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=ego1.Platform, description="LiteX SoC on EGO1.")
    parser.add_target_argument("--flash",               action="store_true",       help="Flash bitstream.")
    parser.add_target_argument("--with-video-terminal", action="store_true",       help="Enable Video Terminal.")
    parser.add_target_argument("--sys-clk-freq",        default=100e6, type=float, help="System clock frequency.")

    args = parser.parse_args()

    soc = BaseSoC(
        sys_clk_freq        = args.sys_clk_freq,
        with_video_terminal = args.with_video_terminal,
        **parser.soc_argdict
    )

    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

    if args.flash:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="flash"))

if __name__ == "__main__":
    main()




#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2023 Charles-Henri Mousset <ch.mousset@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk.
    ("clk25", 0, Pins("K4"), IOStandard("LVCMOS33")),

    # Leds.
    ("user_led", 0, Pins("A18"), IOStandard("LVCMOS33")),

    # RGMII Ethernet (B50612D) PHY 0.
    ("eth_clocks", 0, # U5 is SDIO phy #0
        Subsignal("tx", Pins("A1")),
        Subsignal("rx", Pins("H4")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 0,
        Subsignal("rst_n",   Pins("H2")),
        Subsignal("mdio",    Pins("G2")),
        Subsignal("mdc",     Pins("G1")),
        Subsignal("rx_ctl",  Pins("F1")),
        Subsignal("rx_data", Pins("E3 E2 E1 F3")),
        Subsignal("tx_ctl",  Pins("D1")),
        Subsignal("tx_data", Pins("B2 B1 C2 D2")),
        IOStandard("LVCMOS33")
    ),
    # RGMII Ethernet (B50612D) PHY 1.
    ("eth_clocks", 1, # U9 is SDIO phy #1
        Subsignal("tx", Pins("M6")),
        Subsignal("rx", Pins("L3")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 1,
        Subsignal("rst_n",   Pins("H2")),
        Subsignal("mdio",    Pins("G2")),
        Subsignal("mdc",     Pins("G1")),
        Subsignal("rx_ctl",  Pins("R1")),
        Subsignal("rx_data", Pins("N2 N3 P1 P2")),
        Subsignal("tx_ctl",  Pins("N5")),
        Subsignal("tx_data", Pins("M5 M2 N4 P4")),
        IOStandard("LVCMOS33")
    ),
    # SDRRAM (M12L64322A).
    ("sdram_clock", 0, Pins("E14"), IOStandard("LVCMOS33")),
    ("sdram", 0,
        Subsignal("a", Pins(
            "C20 C19 C13 F13 G13 G15 "
            "F14 F18 E13 E18 C14 A13")),  # Address pin A11 routed but NC on M12L64322A
        Subsignal("dq", Pins(
            "F21 E22 F20 E21 F19 D22 E19 D21 "
            "K21 L21 K22 M21 L20 M22 N20 M20 "
            "B18 D20 A19 A21 A20 B21 C22 B22 "
            "G21 G22 H20 H22 J20 J22 G20 J21 "
            )),
        Subsignal("we_n",  Pins("D17")),
        Subsignal("ras_n", Pins("A14")),
        Subsignal("cas_n", Pins("D14")),
        #Subsignal("cs_n", Pins("")), # GND
        #Subsignal("cke",  Pins("")), # 3V3
        Subsignal("ba",    Pins("D19 B13")),
        #Subsignal("dm",   Pins("")), # GND
        IOStandard("LVCMOS33"),
        Misc("SLEWRATE=FAST")
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ("dimm",
        "- "
        "    GND      5V     GND      5V     GND      5V     GND      5V     GND      5V"  #   1-10
        "    GND      5V      NC      NC ETH1_1P ETH2_1P ETH1_1N ETH2_1N      NC      NC"  #  11-20
        "ETH1_2N ETH2_2N ETH1_2P ETH2_2P      NC      NC ETH1_3P ETH2_3P ETH1_3N ETH2_3N"  #  21-30
        "     NC      NC ETH1_4N ETH2_4N ETH1_4P ETH2_4P      NC      NC     GND     GND"  #  31-40
        "     R2      P5      P6      T6      R6      U7      T1      U6      T3      U5"  #  41-50
        "     T4      V5      T4      U1     GND     GND      U2      H3      U3      J1"  #  51-60
        "     V2      K1      V3      L1      W1      M1      Y1      J2     AA1      K2"  #  61-70
        "    AB1      K3      W2      G3      Y2      J4     AB2      G4     AA3      F4"  #  71-80
        "    AB3      L4      Y3      R3      W4      M3     AA4      V4      Y4      R4"  #  81-90
        "    AB5      T5     AA5      J5      Y6      J6     AB6      W5     AA6      L5"  #  91-100
        "     Y7      L6     AB7      W6     GND     GND     GND     GND     AA8      V7"  # 101-110
        "    AB8     N13      Y8     N14      W7     P15      Y9     P16      V8     R16"  # 111-120
        "     W9     N17      V9     V17     R14     P17     P14     U17     W17     T18"  # 121-130
        "    Y18     R17    AA18     U18     W19     R18    AB18     N18     Y19     R19"  # 131-140
        "   AA19     N19     V18     N15     V19     M16    AB20     M15    AA20     L15"  # 141-150
        "   AA21     L16    AB21     K14     Y21     N22     GND     GND    AB22     J14"  # 151-160
        "    W20     J15     Y22     J19     W21     H13     W22     H14     V20     H17"  # 161-170
        "    V22     H15     U21     G18     U20     G17     T20     G16     P19     F16"  # 171-180
        "    P20     F15     M18     E17     L19     E16     J17     D16     K18     D15"  # 181-190
        "    K19     C18     K16     C17     H18     B20     H19     B17      NC      NC"  # 191-200
    )
]

def pmod_uart(port="P2"):
    if port == "P2":
        return [
        ("serial", 0,
            Subsignal("tx", Pins("dimm:41")),
            Subsignal("rx", Pins("dimm:51")),
            IOStandard("LVCMOS33"))
        ]
    else:
        raise ValueError(f"port {port} not in ['P2']")

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk25"
    default_clk_period = 1e9/25e6

    def __init__(self, toolchain="vivado"):
        Xilinx7SeriesPlatform.__init__(self, "xc7a50tfgg484-1", _io, _connectors, toolchain=toolchain)
        self.toolchain.bitstream_commands = \
            ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]"]
        self.toolchain.additional_commands = \
            ["write_cfgmem -force -format bin -interface spix4 -size 16 "
             "-loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]


    def create_programmer(self, cfg="openocd_xc7_ft2232.cfg"):
        return OpenOCD(cfg, "bscan_spi_xc7a50t.bit")

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request(self.default_clk_name, loose=True), self.default_clk_period)



#!/usr/bin/env python3
#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Hans Baier <hansfbaier@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *

from litex.gen import *

from litex_boards.platforms import terasic_sockit

from litex.soc.cores.clock import CycloneVPLL
from litex.soc.integration.soc_core  import *
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser
from litex.soc.cores.video import VideoVGAPHY

from litex.build.io import DDROutput

from litedram.modules import W9825G6KH6, AS4C32M16
from litedram.phy import HalfRateGENSDRPHY, GENSDRPHY

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq, with_sdram=False, sdram_rate="1:2", with_video_terminal=False):
        self.sdram_rate = sdram_rate
        self.rst    = Signal()
        self.cd_sys = ClockDomain()
        if with_video_terminal:
            self.cd_vga = ClockDomain()
        if with_sdram:
            if sdram_rate == "1:2":
                self.cd_sys2x    = ClockDomain()
                self.cd_sys2x_ps = ClockDomain()
            else:
                self.cd_sys_ps = ClockDomain()

        # Clk / Rst
        clk50 = platform.request("clk50")

        # PLL
        self.pll = pll = CycloneVPLL(speedgrade="-C6")
        self.comb += pll.reset.eq(self.rst)
        pll.register_clkin(clk50, 50e6)
        pll.create_clkout(self.cd_sys, sys_clk_freq)

        if with_video_terminal:
            pll.create_clkout(self.cd_vga, 65e6)

        if with_sdram:
            if sdram_rate == "1:2":
                pll.create_clkout(self.cd_sys2x,    2*sys_clk_freq)
                pll.create_clkout(self.cd_sys2x_ps, 2*sys_clk_freq, phase=180)  # Idealy 90° but needs to be increased.
            else:
                pll.create_clkout(self.cd_sys_ps, sys_clk_freq, phase=90)

        # SDRAM clock
        if with_sdram:
            sdram_clk = ClockSignal("sys2x_ps" if sdram_rate == "1:2" else "sys_ps")
            self.specials += DDROutput(1, 0, platform.request("sdram_clock"), sdram_clk)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=50e6, revision="revd", sdram_rate="1:2", mister_sdram=None,
        with_led_chaser     = True,
        with_video_terminal = False,
        **kwargs):
        platform = terasic_sockit.Platform(revision)

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq,
            with_sdram          = mister_sdram != None,
            sdram_rate          = sdram_rate,
            with_video_terminal = with_video_terminal
        )

        # SoCCore ----------------------------------------------------------------------------------
        if kwargs.get("uart_name", "serial") == "serial":
            kwargs["uart_name"] = "jtag_uart" # Defaults to JTAG-UART.
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on the Terasic SoCKit", **kwargs)

        # SDR SDRAM --------------------------------------------------------------------------------
        if mister_sdram is not None:
            sdrphy_cls = HalfRateGENSDRPHY if sdram_rate == "1:2" else GENSDRPHY
            sdrphy_mod = {"xs_v22": W9825G6KH6, "xs_v24": AS4C32M16}[mister_sdram]
            self.sdrphy = sdrphy_cls(platform.request("sdram"), sys_clk_freq)
            self.add_sdram("sdram",
                phy           = self.sdrphy,
                module        = sdrphy_mod(sys_clk_freq, sdram_rate),
                l2_cache_size = kwargs.get("l2_size", 8192)
            )

        # Video Terminal ---------------------------------------------------------------------------
        if with_video_terminal:
            vga_pads = platform.request("vga")
            self.comb += [ vga_pads.sync_n.eq(0), vga_pads.blank_n.eq(1) ]
            self.specials += DDROutput(i1=1, i2=0, o=vga_pads.clk, clk=ClockSignal("vga"))
            self.videophy = VideoVGAPHY(vga_pads, clock_domain="vga")
            self.add_video_terminal(phy=self.videophy, timings="1024x768@60Hz", clock_domain="vga")

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=terasic_sockit.Platform, description="LiteX SoC on the Terasic SoCKit.")
    parser.add_target_argument("--single-rate-sdram",   action="store_true",      help="Clock SDRAM with 1x the sytem clock (instead of 2x).")
    parser.add_target_argument("--mister-sdram-xs-v22", action="store_true",      help="Use optional MiSTer SDRAM module XS v2.2 on J2 on GPIO daughter card.")
    parser.add_target_argument("--mister-sdram-xs-v24", action="store_true",      help="Use optional MiSTer SDRAM module XS v2.4 on J2 on GPIO daughter card.")
    parser.add_target_argument("--revision",            default="revd",           help="Board revision (revb, revc or revd).")
    parser.add_target_argument("--sys-clk-freq",        default=50e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--with-video-terminal", action="store_true",      help="Enable Video Terminal (VGA).")
    args = parser.parse_args()

    soc = BaseSoC(
        sys_clk_freq        = args.sys_clk_freq,
        revision            = args.revision,
        sdram_rate          = "1:1" if args.single_rate_sdram else "1:2",
        mister_sdram        = "xs_v22" if args.mister_sdram_xs_v22 else "xs_v24" if args.mister_sdram_xs_v24 else None,
        with_video_terminal = args.with_video_terminal,
        **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2015-2019 Florent Kermarrec <florent@enjoy-digital.fr>,
# Copyright (c) 2020 Staf Verhaegen <staf@fibraservi.eu>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *

from litex.gen import *

from litex_boards.platforms import digilent_arty_s7

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser

from litedram.modules import MT41K128M16
from litedram.phy import s7ddrphy

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq):
        self.rst          = Signal()
        self.cd_sys       = ClockDomain()
        self.cd_sys2x     = ClockDomain()
        self.cd_sys4x     = ClockDomain()
        self.cd_sys4x_dqs = ClockDomain()
        self.cd_idelay    = ClockDomain()

        # # #

        self.pll = pll = S7PLL(speedgrade=-1)
        self.comb += pll.reset.eq(~platform.request("cpu_reset") | self.rst)
        pll.register_clkin(platform.request("clk100"), 100e6)
        pll.create_clkout(self.cd_sys,       sys_clk_freq)
        pll.create_clkout(self.cd_sys2x,     2*sys_clk_freq)
        pll.create_clkout(self.cd_sys4x,     4*sys_clk_freq)
        pll.create_clkout(self.cd_sys4x_dqs, 4*sys_clk_freq, phase=90)
        pll.create_clkout(self.cd_idelay,    200e6)
        platform.add_false_path_constraints(self.cd_sys.clk, pll.clkin) # Ignore sys_clk to pll.clkin path created by SoC's rst.

        self.idelayctrl = S7IDELAYCTRL(self.cd_idelay)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, variant="s7-50", sys_clk_freq=100e6,
        with_spi_flash  = False,
        with_led_chaser = True,
        **kwargs):
        platform = digilent_arty_s7.Platform(variant=variant)

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq)

        # SoCCore ----------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on Arty S7", **kwargs)

        # DDR3 SDRAM -------------------------------------------------------------------------------
        if not self.integrated_main_ram_size:
            self.ddrphy = s7ddrphy.A7DDRPHY(platform.request("ddram"),
                memtype        = "DDR3",
                nphases        = 4,
                sys_clk_freq   = sys_clk_freq)
            self.add_sdram("sdram",
                phy           = self.ddrphy,
                module        = MT41K128M16(sys_clk_freq, "1:4"),
                l2_cache_size = kwargs.get("l2_size", 8192)
            )

        # SPI Flash --------------------------------------------------------------------------------
        if with_spi_flash:
            from litespi.modules import S25FL128S
            from litespi.opcodes import SpiNorFlashOpCodes as Codes
            self.add_spi_flash(mode="4x", module=S25FL128S(Codes.READ_1_1_4), with_master=True)

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=digilent_arty_s7.Platform, description="LiteX SoC on Arty S7.")
    parser.add_target_argument("--variant",        default="s7-50",           help="Board variant (s7-50 or s7-25).")
    parser.add_target_argument("--sys-clk-freq",   default=100e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--with-spi-flash", action="store_true",       help="Enable SPI Flash (MMAPed).")
    args = parser.parse_args()

    soc = BaseSoC(
        variant        = args.variant,
        sys_clk_freq   = args.sys_clk_freq,
        with_spi_flash = args.with_spi_flash,
        **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Paul Sajna <sajattack@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.altera import AlteraPlatform
from litex.build.altera.programmer import USBBlaster

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk50", 0, Pins("V11"), IOStandard("3.3-V LVTTL")),
    ("clk50", 1, Pins("Y13"), IOStandard("3.3-V LVTTL")),
    ("clk50", 2, Pins("E11"), IOStandard("3.3-V LVTTL")),

    # Leds
    ("user_led", 0, Pins("W15"),  IOStandard("3.3-V LVTTL")),
    ("user_led", 1, Pins("AA24"), IOStandard("3.3-V LVTTL")),
    ("user_led", 2, Pins("V16"),  IOStandard("3.3-V LVTTL")),
    ("user_led", 3, Pins("V15"),  IOStandard("3.3-V LVTTL")),
    ("user_led", 4, Pins("AF26"), IOStandard("3.3-V LVTTL")),
    ("user_led", 5, Pins("AE26"), IOStandard("3.3-V LVTTL")),
    ("user_led", 6, Pins("Y16"),  IOStandard("3.3-V LVTTL")),
    ("user_led", 7, Pins("AA23"), IOStandard("3.3-V LVTTL")),

    # Buttons
    ("key", 0, Pins("AH17"), IOStandard("3.3-V LVTTL")),
    ("key", 1, Pins("AH16"), IOStandard("3.3-V LVTTL")),

    # Switches
    ("user_sw", 0, Pins("Y24"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 1, Pins("W24"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 2, Pins("W21"), IOStandard("3.3-V LVTTL")),
    ("user_sw", 3, Pins("W20"), IOStandard("3.3-V LVTTL")),

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("AH9"),  IOStandard("3.3-V LVTTL")), # User I/O port on Mister
        Subsignal("rx", Pins("AG11"), IOStandard("3.3-V LVTTL"))  # User I/O port on Mister
    ),

    # Serial
    ("serial", 1,
        Subsignal("tx", Pins("AF13"), IOStandard("3.3-V LVTTL")),  # Arduino_IO1
        Subsignal("rx", Pins("AG13"), IOStandard("3.3-V LVTTL"))   # Arduino_IO0
    ),

    # Accelerometer
    ("acc", 0,
        Subsignal("int",  Pins("A17")),
        Subsignal("sclk", Pins("C18")),
        Subsignal("sdat", Pins("A19")),
        IOStandard("3.3-V LVTTL")
    ),

    # ADC
    ("adc", 0,
        Subsignal("convst", Pins("U9")),
        Subsignal("sclk",   Pins("V10")),
        Subsignal("sdi",    Pins("AC4")),
        Subsignal("sdo",    Pins("AD4")),
        IOStandard("3.3-V LVTTL")
    ),

    # HDMI
    ("hdmi", 0,
        Subsignal("tx_d_r", Pins("AD12 AE12 W8 Y8 AD11 AD10 AE11 Y5")),
        Subsignal("tx_d_g", Pins("AF10 Y4 AE9 AB4 AE7 AF6 AF8 AF5")),
        Subsignal("tx_d_b", Pins("AE4 AH2 AH4 AH5 AH6 AG6 AF9 AE8")),
        Subsignal("tx_clk", Pins("AG5")),
        Subsignal("tx_de",  Pins("AD19")),
        Subsignal("tx_hs",  Pins("T8")),
        Subsignal("tx_vs",  Pins("V13")),
        Subsignal("tx_int", Pins("AF11")),
        Misc("FAST_OUTPUT_REGISTER ON"),
        IOStandard("3.3-V LVTTL")
    ),

    # I2C
    ("i2c", 0,
        Subsignal("scl",    Pins("U10")),
        Subsignal("sda",    Pins("AA4")),
        IOStandard("3.3-V LVTTL")
    ),

    # I2S
    ("i2s", 0,
        Subsignal("i2s",   Pins("T13")),
        Subsignal("mclk",   Pins("U11")),
        Subsignal("lrclk",  Pins("T11")),
        Subsignal("sclk",   Pins("T12")),
        IOStandard("3.3-V LVTTL")
    ),
]

# MiSTer extension board (https://github.com/MiSTer-devel/Main_MiSTer/wiki) ------------------------

_mister_sdram_module_io = [
    # SDR SDRAM
    ("sdram_clock", 0, Pins("AD20"), IOStandard("3.3-V LVTTL")),
    ("sdram", 0,
        Subsignal("cke", Pins("AG10")),
        Subsignal("a",   Pins(
            "Y11 AA26 AA13 AA11 W11 Y19 AB23 AC23",
            "AC22 C12 AB26 AD17 D12")),
        Subsignal("dq",  Pins(
            "E8    V12 D11  W12 AH13  D8 AH14 AF7",
            "AE24 AD23 AE6 AE23 AG14 AD5  AF4 AH3"),
            Misc("FAST_OUTPUT_ENABLE_REGISTER ON"),
            Misc("FAST_INPUT_REGISTER ON")),
        Subsignal("ba",    Pins("Y17 AB25")),
        Subsignal("cas_n", Pins("AA18")),
        Subsignal("cs_n",  Pins("Y18")),
        Subsignal("ras_n", Pins("W14")),
        Subsignal("we_n",  Pins("AA19")),
        Misc("CURRENT_STRENGTH_NEW \"MAXIMUM CURRENT\""),
        Misc("FAST_OUTPUT_REGISTER ON"),
        Misc("ALLOW_SYNCH_CTRL_USAGE OFF"),
        IOStandard("3.3-V LVTTL"),
    ),

    # SDCard
    ("spisdcard", 0,
        Subsignal("clk",  Pins("AH26")),
        Subsignal("cs_n", Pins("AF28")),
        Subsignal("mosi", Pins("AF27"), Misc("WEAK_PULL_UP_RESISTOR ON")),
        Subsignal("miso", Pins("AF25"), Misc("WEAK_PULL_UP_RESISTOR ON")),
        IOStandard("3.3-V LVTTL")
    ),

    ("sdcard", 0,
        Subsignal("data", Pins("AF25 AF23 AD26 AF28"), Misc("WEAK_PULL_UP_RESISTOR ON")),
        Subsignal("cmd", Pins("AF27"), Misc("WEAK_PULL_UP_RESISTOR ON")),
        Subsignal("clk", Pins("AH26")),
        Subsignal("cd", Pins("AH7")),
        Misc("FAST_OUTPUT_REGISTER ON"),
        IOStandard("3.3-V LVTTL"),
    ),

    # Outputs
    ("mister_outputs", 0,
        Subsignal("led_user",  Pins("Y15")),
        Subsignal("led_hdd",   Pins("AA15")),
        Subsignal("led_power", Pins("AG28")),
        IOStandard("3.3-V LVTTL")
    ),

    # VGA
    ("vga", 0,
        Subsignal("en",      Pins("AH27")),
        Subsignal("hsync_n", Pins("AH22")),
        Subsignal("vsync_n", Pins("AG24")),
        Subsignal("r", Pins("AE17 AE20 AF20 AH18 AH19 AF21")),
        Subsignal("g", Pins("AE19 AG15 AF18 AG18 AG19 AG20")),
        Subsignal("b", Pins("AG21 AA20 AE22 AF22 AH23 AH21")),
        IOStandard("3.3-V LVTTL")
    ),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(AlteraPlatform):
    default_clk_name   = "clk50"
    default_clk_period = 1e9/50e6

    def __init__(self, toolchain="quartus"):
        AlteraPlatform.__init__(self, "5CSEBA6U23I7", _io, toolchain=toolchain)
        self.add_extension(_mister_sdram_module_io)

    def create_programmer(self):
        return USBBlaster(cable_name="DE-SoC", device_id=2)

    def do_finalize(self, fragment):
        AlteraPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk50", 0, loose=True), 1e9/50e6)
        self.add_period_constraint(self.lookup_request("clk50", 1, loose=True), 1e9/50e6)
        self.add_period_constraint(self.lookup_request("clk50", 2, loose=True), 1e9/50e6)
